<?php
/**
 * The automation class
 *
 * Used to define custom automation functions used for displaying and handling automation steps
 *
 *
 * @since      9.7
 * @package    CartBounty Pro - Save and recover abandoned carts for WooCommerce
 * @subpackage CartBounty Pro - Save and recover abandoned carts for WooCommerce/includes
 * @author     Streamline.lv
 */

class CartBounty_Pro_Automation{

	/**
	 * Checking if automation enabled. At least one automation template should be enabled
	 *
	 * @since    9.7
	 * @return   string
     * @param    boolean    $enabled    		  Wheather automation has been enabled or not
	 */
	public function display_automation_status( $enabled ) {
		if( $enabled ){
			$status = sprintf( '<span class="status active">%s</span>', esc_html__( 'Active', 'woo-save-abandoned-carts' ) );

		}else{
			$status = sprintf( '<span class="status inactive">%s</span>', esc_html__( 'Disabled', 'woo-save-abandoned-carts' ) );
		}

		echo $status;
	}

	/**
	 * Check if a given automation step is enabled
	 * By default will return all active automation steps
	 *
	 * @since    9.7
	 * @return   array or boolean
	 * @param    string       $recovery				Recovery option
	 * @param    integer      $automation			Automation step. If not provided, an array of enabled automation steps returned
	 * @param    integer      $single				Weather we want to return single value or not
	 */
	public function get_active_steps( $recovery, $automation = false, $single = false ){
		$result = false;

		if( $recovery == 'wordpress' ){
			$option = 'cartbounty_pro_automation_steps';
			
		}elseif( $recovery == 'bulkgate' ){
			$option = 'cartbounty_pro_bulkgate_steps';
		}

		$this->restore_steps( $recovery, $option );
		$automation_steps = get_option( $option );

		if( $single ){ //If we want to know if a specific automation step is active
			$active_steps = $automation_steps[$automation];
			if( isset( $active_steps['enabled'] ) ){
				$result = true;
			}

		}else{
			$result = array();
			foreach ( $automation_steps as $key => $step ) {
				if( isset( $step['enabled'])){
					$result[] = $key;
				}
			}
		}
		return $result;
	}

	/**
	 * Checking total number of activated steps and determines which step should be accessible
	 * Returns a class if the step is should not be accessible
	 * First automation step is always accessible
	 *
	 * @since    9.7
	 * @return   string
	 * @param    string     $recovery				Recovery option
	 * @param    integer    $automation				Automation number
	 */
	public function check_if_accessible( $recovery, $automation = false ){
		$class = '';
		$active_steps = $this->get_active_steps( $recovery );
		
		if( $automation != 0 ){ //If we are not looking at first automation step
			if( $automation > count( $active_steps ) ){
				$class = " cartbounty-pro-step-disabled";
			}
		}

		echo $class;
	}

	/**
	* Method validates automation step data and disables steps which should not be active
	* Deactivating 2nd and 3rd automation steps in case 1st step disabled
	* Deactivating 3rd automation step in case 2nd step disabled
	*
	* @since    9.7
	*/
	public function validate_automation_steps(){
		if( isset( $_POST['cartbounty_pro_automation_steps'] ) ){
			$steps = $_POST['cartbounty_pro_automation_steps'];
			$option = 'cartbounty_pro_automation_steps';

		}elseif( isset( $_POST['cartbounty_pro_bulkgate_steps'] ) ){
			$steps = $_POST['cartbounty_pro_bulkgate_steps'];
			$option = 'cartbounty_pro_bulkgate_steps';

		}else{ //Exit in case the automation step data is not present
			return;
		}

		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$disabled_steps = array();

		foreach ( $steps as $key => $step ) {
			if( !isset( $step['enabled'] ) ){
				$disabled_steps[] = $key; //Add current step to disabled steps array
			}

			//Sanitizing Subject
			if( isset( $step['subject'] ) ){
				$steps[$key]['subject'] = sanitize_text_field( $step['subject'] );
			}
			//Sanitizing Heading
			if( isset( $step['heading'] ) ){
				$steps[$key]['heading'] = $admin->sanitize_field( $step['heading'] );
			}
			//Sanitizing Content
			if( isset( $step['content'] ) ){
				$steps[$key]['content'] = $admin->sanitize_field( $step['content'] );
			}
			//Sanitizing Coupon description
			if( isset( $step['coupon_description'] ) ){
				$steps[$key]['coupon_description'] = $admin->sanitize_field( $step['coupon_description'] );
			}
			//Sanitizing Coupon prefix
			if( isset( $step['coupon_prefix'] ) ){
				$steps[$key]['coupon_prefix'] = $admin->sanitize_field( $step['coupon_prefix'] );
			}
		}

		foreach ( $disabled_steps as $key => $disabled_step ) {
			if( $disabled_step == 0 ){ //If first step is disabled, deactivate 2nd and 3rd steps
				unset( $steps[1]['enabled'] );
				unset( $steps[2]['enabled'] );

			}elseif( $disabled_step == 1 ){ //If second step is disabled, deactivate 3rd step
				unset( $steps[2]['enabled'] );
			}
		}

		update_option( $option, $steps );
	}

	/**
	* Method restores automation steps in case they are cleared, deleted or do not exist
	*
	* @since    9.7
	* @param    string		$recovery			Recovery option
	* @param    string		$option   		    Option name
	*/
	private function restore_steps( $recovery, $option ){
		$automation_steps = get_option( $option );
		
		if( empty( $automation_steps ) ){
			if( $recovery == 'wordpress' ){
				$wordpress = new CartBounty_Pro_WordPress();
				$coupons = new CartBounty_Pro_Coupons();
				update_option( $option,
					array(
						array(
							'subject' => $wordpress->get_defaults( 'subject', 0 ),
							'heading' => $wordpress->get_defaults( 'heading', 0 ),
							'content' => $wordpress->get_defaults( 'content', 0 ),
							'coupon_description' => $coupons->get_defaults( 'coupon_description', 0 ),
							'include_image' => true,
							'coupon_expiry' => 10080
						),
						array(
							'subject' => $wordpress->get_defaults( 'subject', 1 ),
							'heading' => $wordpress->get_defaults( 'heading', 1 ),
							'content' => $wordpress->get_defaults( 'content', 1 ),
							'coupon_description' => $coupons->get_defaults( 'coupon_description', 1 ),
							'include_image' => true,
							'coupon_expiry' => 10080
						),
						array(
							'subject' => $wordpress->get_defaults( 'subject', 2 ),
							'heading' => $wordpress->get_defaults( 'heading', 2 ),
							'content' => $wordpress->get_defaults( 'content', 2 ),
							'coupon_description' => $coupons->get_defaults( 'coupon_description', 2 ),
							'include_image' => true,
							'coupon_expiry' => 10080
						)
					)
				);

			}elseif( $recovery == 'bulkgate' ){
				$bulkgate = new CartBounty_Pro_BulkGate();
				update_option( $option,
					array(
						array(
							'content' => $bulkgate->get_defaults( 'content', 0 ),
							'coupon_expiry' => 10080
						),
						array(
							'content' => $bulkgate->get_defaults( 'content', 1 ),
							'coupon_expiry' => 10080
						),
						array(
							'content' => $bulkgate->get_defaults( 'content', 2 ),
							'coupon_expiry' => 10080
						)
					)
				);
			}
		}
	}

	/**
     * Update abandoned cart last sent time, completed steps and if the automation is completed or not
     *
     * @since    9.7
     * @param    object     $cart   		    Cart data
     * @param    string     $current_time       Current time
     * @param    string     $recovery			Recovery option
     */
	public function update_cart( $cart, $current_time, $recovery ){
		global $wpdb;
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$complete = false;
		$active_steps = $this->get_active_steps( $recovery );

		if( $recovery == 'wordpress' ){
			$completed_steps = $cart->wp_steps_completed;
			$field_steps_completed = 'wp_steps_completed';
			$field_last_sent = 'wp_last_sent';
			$field_completed = 'wp_complete';

		}elseif( $recovery == 'bulkgate' ){
			$completed_steps = $cart->sms_steps_completed;
			$field_steps_completed = 'sms_steps_completed';
			$field_last_sent = 'sms_last_sent';
			$field_completed = 'sms_complete';
		}

		if( $completed_steps >= ( count( $active_steps ) - 1 ) ){ //Checking if this is the last automation step to mark the automation as complete for the current cart
			$complete = true;
		}

		$wpdb->query(
			$wpdb->prepare(
				"UPDATE {$cart_table}
				SET $field_steps_completed = %d,
				$field_last_sent = %s,
				$field_completed = %s
				WHERE id = %d",
				$completed_steps + 1,
				$current_time,
				$complete,
				$cart->id
			)
		);
	}

	/**
     * Method returns available time intervals alongside a selected interval for a given automation or just the name of the selected interval if $selected_name set to true
     *
     * @since    9.7
     * @return   array or string
     * @param    string     $recovery			Recovery option
     * @param    integer    $automation			Automation number
     * @param    boolean    $selected_name		Should just the name of the selected Interval be returned
     */
	public function get_intervals( $recovery, $automation = false, $selected_name = false ){
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$minutes = array( 5, 10, 15, 20, 25, 30, 40, 50, 60, 120, 180, 240, 300, 360, 420, 480, 540, 600, 660, 720, 1080, 1440, 2880, 4320, 5760, 7200, 8640, 10080 ); //Defining array of minutes
		
		if( $recovery == 'wordpress' ){
			$wordpress = new CartBounty_Pro_WordPress();
			$automation_steps = get_option( 'cartbounty_pro_automation_steps' );
			$selected_interval = $wordpress->get_defaults( 'interval', $automation );

		}elseif( $recovery == 'bulkgate' ){
			$bulkgate = new CartBounty_Pro_BulkGate();
			$automation_steps = get_option( 'cartbounty_pro_bulkgate_steps' );
			$selected_interval = $bulkgate->get_defaults( 'interval', $automation );
		}

		if( isset( $automation_steps[$automation] ) ){
			$automation_step = $automation_steps[$automation];
		}

		if( !empty( $automation_step['interval'] ) ){ //If custom interval is not set, fallback to default
			$selected_interval = $automation_step['interval'];
		}

		$intervals = $admin->prepare_time_intervals( $minutes );

		if( $selected_name ){ //In case just the selected name should be returned
			return $intervals[$selected_interval];

		}else{
			return array(
				'intervals'  => $intervals,
				'selected'   => $selected_interval
			);
		}
	}

	/**
	 * Method displays available time intervals at which we are sending out abandoned carts
	 *
	 * @since    9.7
	 * @return   string
	 * @param    string     $recovery			Recovery option
	 * @param    integer    $automation			Automation number
	 */
	public function display_intervals( $recovery, $automation = false ){
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$array = $this->get_intervals( $recovery, $automation );

		if( $recovery == 'wordpress' ){
			$name = 'cartbounty_pro_automation_steps';

		}elseif( $recovery == 'bulkgate' ){
			$name = 'cartbounty_pro_bulkgate_steps';
		}

		echo '<select id="cartbounty-pro-automation-interval-'. esc_attr( $automation ) .'" class="cartbounty-pro-select" name="'. $name .'['. esc_attr( $automation ) .'][interval]" autocomplete="off" '. $admin->disable_field() .'>';
			foreach( $array['intervals'] as $key => $interval ){
				echo "<option value='". esc_attr( $key ) ."' ". selected( $array['selected'], $key, false ) .">". esc_html( $interval ) ."</option>";
			}
		echo '</select>';
	}

	/**
     * Handle stats update according to the action that has been recorded.
     *
     * @since    9.7
     * @param    string     $action   			Action type that must be added
     * @param    integer    $cart_id   			Cart ID
     * @param    integer    $step_nr            Automation step number
     * @param    string     $recovery			Recovery option
     */
	public function handle_message_stats( $action, $cart_id, $step_nr, $recovery ){
		if( $recovery == 'wordpress' ){
			if($action == 'open'){
				$update = $this->update_message_stats( $action, $cart_id, $step_nr, $recovery );

				if( $update ){
					$this->increase_message_stats( $step_nr, $action, $recovery );
				}

			}elseif( $action == 'click' ){
				$update = $this->update_message_stats( $action, $cart_id, $step_nr, $recovery );

				if( $update ){
					$this->increase_message_stats( $step_nr, $action, $recovery );
				}

				$action = 'open';
				$update = $this->update_message_stats( $action, $cart_id, $step_nr, $recovery );

				if( $update ){
					$this->increase_message_stats( $step_nr, $action, $recovery );
				}
			}

		}elseif( $recovery == 'bulkgate' ){
			if( $action == 'click' ){
				$update = $this->update_message_stats( $action, $cart_id, $step_nr, $recovery );

				if( $update ){
					$this->increase_message_stats( $step_nr, $action, $recovery );
				}
			}
		}
	}

	/**
     * Update message statistics in the database according to the action.
     * In case the message table has duplicate messages sent in the same step, we update only the last one not to disrupt any statistics
     *
     * @since    9.7
     * @return   boolean or integer             False or updated row count
     * @param    string     $action   			Action type that must be added (open, click)
     * @param    integer    $cart_id   			Cart ID
     * @param    integer    $step_nr            Automation step number
     * @param    string     $recovery			Recovery option
     */
	public function update_message_stats( $action, $cart_id, $step_nr, $recovery ){
		global $wpdb;

		if( $recovery == 'wordpress' ){
			$wordpress = new CartBounty_Pro_WordPress();
			//Making sure that the email table exists or gets created
			if(!$wordpress->email_table_exist()){
				$wordpress->create_email_table();
			}
			$table_name = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME_EMAILS;

		}elseif( $recovery == 'bulkgate' ){
			$bulkgate = new CartBounty_Pro_BulkGate();
			//Making sure that message table exists or gets created
			if( !$bulkgate->message_table_exist() ){
				$bulkgate->create_message_table();
			}
			$table_name = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME_MESSAGES;
		}

		$update = $wpdb->query(
			$wpdb->prepare(
				"UPDATE $table_name
				SET $action = %d
				WHERE cart = %d
				AND step = %d
				ORDER BY id DESC
				LIMIT %d",
				1,
				$cart_id,
				$step_nr,
				1
			)
		);

		return $update;
	}

	/**
     * Increase text message stats of specified automation (sends, opens, clicks)
     *
     * @since    9.7
     * @param    integer    $step_nr			Automation step number
     * @param    string     $action				Action type that needs to be updated
     * @param    string     $recovery			Recovery option
     */
	public function increase_message_stats( $step_nr, $action, $recovery ){
		if( $recovery == 'wordpress' ){

			switch ( $action ) {
				case 'send':
					$option = 'cartbounty_pro_automation_sends';
					break;

				case 'open':
					$option = 'cartbounty_pro_automation_opens';
					break;

				case 'click':
					$option = 'cartbounty_pro_automation_clicks';
					break;
			}

		}elseif( $recovery == 'bulkgate' ){

			switch ( $action ) {
				case 'send':
					$option = 'cartbounty_pro_bulkgate_sends';
					break;

				case 'click':
					$option = 'cartbounty_pro_bulkgate_clicks';
					break;
			}
		}

		$message_stats = get_option( $option );
		if( isset( $message_stats[$step_nr] ) ){ //If we already have previous stats for the current step
			$message_stats[$step_nr] = $message_stats[$step_nr] + 1;

		}else{ //If this is the first time we track this
			$message_stats[$step_nr] = 1;
		}
		update_option( $option, $message_stats );
	}

	/**
     * Retrieve sent message history of a given abandoned cart
     *
     * @since    9.7
     * @return   array
     * @param    string     $recovery			Recovery option
     * @param    integer    $cart_id   			Cart ID
     */
	public function get_message_history( $recovery, $cart_id = false ){
		global $wpdb;

		if( $recovery == 'wordpress' ){
			$wordpress = new CartBounty_Pro_WordPress();
			//Making sure that the email table exists or gets created
			if(!$wordpress->email_table_exist()){
				$wordpress->create_email_table();
			}
			$table_name = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME_EMAILS;

		}elseif( $recovery == 'bulkgate' ){
			$bulkgate = new CartBounty_Pro_BulkGate();
			//Making sure that message table exists or gets created
			if( !$bulkgate->message_table_exist() ){
				$bulkgate->create_message_table();
			}
			$table_name = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME_MESSAGES;
		}

		$messages = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT *
				FROM {$table_name}
				WHERE cart = %s
				ORDER BY step ASC",
				$cart_id
			)
		);
		return $messages;
	}

	/**
     * Output a list of sent messages for a given cart
     *
     * @since    9.7
     * @return   array
     * @param    string     $recovery			Recovery option
     * @param    integer    $cart_id   			Cart ID
     */
	public function display_message_history( $recovery, $cart_id = false ){
		$messages = $this->get_message_history( $recovery, $cart_id );
		$output = '';

		if( $messages ){
			$output .= '<em class="cartbounty-pro-message-history-list">';
			foreach( $messages as $key => $message ){
				$step = $message->step + 1; //Adding + 1 as step numbering starts from 0
				$time = new DateTime( $message->time );
				$output .= '<i class="cartbounty-pro-message-history-item"><i class="cartbounty-pro-automation-number">'. esc_html( $step ) .'</i>' . esc_html( $time->format( 'M d, Y H:i' ) ) . '</i>';
			}
			$output .= '</em>';
		}
		return $output;
	}

	/**
	 * Return abandoned cart stats.
	 * If a specific step number is not requested - will return total stats for all of the steps.
	 *
	 * @since    9.7
	 * @return   integer or array
	 * @param    string     $recovery			Recovery option
	 * @param    string     $action				Action type that needs to be returned
	 * @param    integer    $step_nr			Automation step number
	 * @param    boolean    $single				Whether a value for a specified step must be returned
	 */
	public function get_stats( $recovery, $action, $step_nr = false, $single = false ){
		$count = 0;

		if( $recovery == 'wordpress' ){
			switch ( $action ) {
				case 'send':
					$option = 'cartbounty_pro_automation_sends';
					break;

				case 'open':
					$option = 'cartbounty_pro_automation_opens';
					break;

				case 'click':
					$option = 'cartbounty_pro_automation_clicks';
					break;
			}

		}elseif( $recovery == 'bulkgate' ){
			switch ( $action ) {
				case 'send':
					$option = 'cartbounty_pro_bulkgate_sends';
					break;

				case 'click':
					$option = 'cartbounty_pro_bulkgate_clicks';
					break;
			}
		}

		$message_stats = get_option( $option );

		if( $single ){ //If stats for a specific step number is requested
			if( isset( $message_stats[$step_nr] ) ){
				$count = $message_stats[$step_nr];
			}

		}else{ //Counting message stats across all automation steps
			if( is_array( $message_stats ) ){
				foreach ( $message_stats as $key => $stat ) {
					$count = $count + $stat;
				}
			}
		}
		return $count;
	}

	/**
	 * Return abandoned cart message stats in percents
	 *
	 * @since    9.7
	 * @return   string
	 * @param    string     $recovery			Recovery option
	 * @param    string     $action				Action type that needs to be returned
	 * @param    integer    $step_nr			Automation step number
	 */
	public function get_stats_percentage( $recovery, $action, $step_nr ){
		$percents = $this->calculate_open_click_rate( $action, $step_nr, $recovery );
		return number_format( ( float )$percents, $decimals = 1, '.', '' ) . '%';
	}

	/**
	 * Return message open rates and click-through rates for the given step
	 *
	 * @since    9.7
	 * @return   float
	 * @param    string     $action				Action type that needs to be returned
	 * @param    integer    $step_nr			Automation step number
	 * @param    string     $recovery			Recovery option
	 */
	public function calculate_open_click_rate( $action, $step_nr, $recovery ){
		$rate = 0;
		$total_actions = $this->get_stats( $recovery, $action, $step_nr, $single = true ); //Get total clicks for the step
		$total_sends = $this->get_stats( $recovery, 'send', $step_nr, $single = true ); //Get total sends for the step

		if( $total_sends != 0 ){ //Prevent division by zero
			if( $total_sends < $total_actions ){ //If actions exceed sent message amount. Prevents from displaying results like 300%
				$total_sends = $total_actions;
			}
			$rate = $total_actions / $total_sends * 100;
		}
		return $rate;
	}

	/**
	 * Reset stats by deleting stats options
	 *
	 * @since    9.7
	 * @return   string
	 * @param    string     $recovery			Recovery option
	 */
	public function reset_stats( $recovery ){
		if( $recovery == 'wordpress' ){
			delete_option( 'cartbounty_pro_automation_sends' );
			delete_option( 'cartbounty_pro_automation_opens' );
			delete_option( 'cartbounty_pro_automation_clicks' );

		}elseif( $recovery == 'bulkgate' ){
			delete_option( 'cartbounty_pro_bulkgate_sends' );
			delete_option( 'cartbounty_pro_bulkgate_clicks' );
		}
	}

	/**
	* Show if coupon is enabled for a specific step 
	*
	* @since    9.7
	* @return   HTML
	* @param    string     $recovery			Recovery option
	* @param    integer    $step_nr				Automation step number
	*/
	public function display_coupon_status( $recovery, $step_nr ){
		if( $recovery == 'wordpress' ){
			$name = 'cartbounty_pro_automation_steps';

		}elseif( $recovery == 'bulkgate' ){
			$name = 'cartbounty_pro_bulkgate_steps';
		}

		$output = '';
		$coupons = new CartBounty_Pro_Coupons();
		$coupon_enabled = $coupons->coupon_enabled( $recovery, $name, $step_nr );
		
		if( $coupon_enabled ){ //If coupon enabled
			$output = sprintf( '<div class="status-item-container"><span class="cartbounty-pro-tooltip">%s</span><span class="status coupon-active">%s</span></div>', esc_html__( 'Coupon enabled', 'woo-save-abandoned-carts' ), 'CP' );

		}
		echo $output;
	}

	/**
	 * Return abandoned carts waiting in the given automation step queue.
	 * Queue is counted for both enabled and disabled steps.
	 *
	 * @since    9.7
	 * @return   string
	 * @param    string     $recovery			Recovery option
	 * @param    integer    $step_nr			Automation step number
	 */
	public function get_queue( $recovery, $step_nr = false ) {
		global $wpdb;
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$time = $admin->get_time_intervals();
		$count = 0;
		$phone_consent_query = '';

		if( $recovery == 'wordpress' ){
			$automation_steps = get_option( 'cartbounty_pro_automation_steps' );
			$field_data_field = 'email';
			$field_unsubscribed = 'wp_unsubscribed';
			$field_excluded = "wp_excluded";
			$field_complete = 'wp_complete';
			$field_steps_completed = 'wp_steps_completed';

		}elseif( $recovery == 'bulkgate' ){
			$automation_steps = get_option( 'cartbounty_pro_bulkgate_steps' );
			$field_data_field = 'phone';
			$field_unsubscribed = 'sms_unsubscribed';
			$field_excluded = "sms_excluded";
			$field_complete = 'sms_complete';
			$field_steps_completed = 'sms_steps_completed';

			if( $admin->phone_consent_enabled() ){ //Check if phone consent enabled to query only carts with given phone number usage consent
				$phone_consent_query = 'AND phone_consent = 1';
			}
		}

		//Retrieving all abandoned carts that are eligible for recovery
		//Excluding finished automations, unsubscried carts and excluded carts (ones that have been manually removed from automation)
		$carts = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT id
				FROM {$cart_table}
				WHERE (type = %d OR type = %d) AND
				$field_data_field != '' AND
				cart_contents != '' AND
				$field_unsubscribed != 1 AND
				paused != 1 AND
				$field_excluded != 1 AND
				$field_complete != 1 AND
				$field_steps_completed = %d AND
				time < %s AND
				time > %s
				$phone_consent_query",
				$admin->get_cart_type('abandoned'),
				$admin->get_cart_type('recovered_pending'),
				$step_nr,
				$time['cart_abandoned'],
				$time['maximum_sync_period']
			)
		);

		foreach ( $automation_steps as $key => $step ) { //Looping through automation steps
			if( $step_nr == $key ){ //If current step must be complete
				$count = count( $carts );
			}
		}
		return $count;
	}

	/**
	* Reset automation step stats
	*
	* @since    9.4.3
	*/
	public function handle_stats_reset(){
		$handle = 'cartbounty_pro_reset_stats';

		if( isset( $_GET[$handle] ) ){ //Check if we should reset stats
			check_admin_referer( 'cartbounty-pro-reset-stats-nonce', $handle ); //Exit in case security check is not passed

			if( isset( $_GET['section'] ) ){
				$automation = new CartBounty_Pro_Automation();
				$automation->reset_stats( $_GET['section'] );
				wp_redirect( '?page='. esc_attr( $_REQUEST['page'] ) .'&tab='. esc_attr( $_REQUEST['tab'] ) .'&section='. esc_attr( $_REQUEST['section'] ), '303' ); //Redirect to the same page without the get parameters in the link

			}
		}
	}

	/**
	* Add click-through rate tracking URL to existing recovery link
	*
	* @since    9.4.3
	* @return   string
	* @param    string    $url					Recovery link
	* @param    integer   $step_nr				Automation step number
	* @param    string    $recovery				Recovery option
	*/
	public function add_click_through_tracking( $url, $step_nr, $recovery ){
		if( $recovery == 'wordpress' ){
			$url = $url . '&cartbounty-pro-step=' . $step_nr;
			
		}elseif( $recovery == 'bulkgate' ){ //For the purposes of keeping the link short, the last symbol of the link determines automation step
			$url = $url . $step_nr;
		}

		return $url;
	}

	/**
	 * Get dynamic content input fields
	 *
	 * @since    9.7.3
	 * @return   array
	 * @param    integer    $step_nr			Automation step number
	 * @param    array      $step				Step data
	 * @param    string     $field_name			Name of the field to retrieve
	 * @param    string     $recovery			Recovery option
	 */
	function get_content_input_fields( $step_nr, $step, $field_name, $recovery ){
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$field = array();

		if( $recovery == 'wordpress' ){
			$wordpress = new CartBounty_Pro_WordPress();
			$field['name'] = $wordpress->get_defaults( $field_name . '_name', $step_nr ); //This is necessary for WPML
			$field['value'] = $wordpress->get_defaults( $field_name, $step_nr );

		}elseif( $recovery == 'bulkgate' ){
			$bulkgate = new CartBounty_Pro_BulkGate();
			$field['name'] = $bulkgate->get_defaults( $field_name . '_name', $step_nr ); //This is necessary for WPML
			$field['value'] = $bulkgate->get_defaults( $field_name, $step_nr );
		}

		if( isset( $step[$field_name] ) ){
			if( trim( $step[$field_name] ) != '' ){ //If the value is not empty and does not contain only whitespaces
				$field['value'] = $admin->sanitize_field( $step[$field_name] );
			}
		}
		return $field;
	}
}