<?php
/**
* The Coupon class.
*
* Used to define functions related with coupon creation and management
*
*
* @since      9.4
* @package    CartBounty Pro - Save and recover abandoned carts for WooCommerce
* @subpackage CartBounty Pro - Save and recover abandoned carts for WooCommerce/includes
* @author     Streamline.lv
*/
class CartBounty_Pro_Coupons{

	/**
	* Run the coupon functionality and returns array consisting of coupon code with coupon expiration time
	* Returns false in case the coupon could not be generated / used or a coupon code in case the coupon code is valid for the given cart and can be used
	*
	* @since    9.4
	* @return   boolean or array
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    integer    $step_nr              Automation step number
	* @param    object     $cart                 Cart data
	*/
	public function get_coupon( $recovery, $option, $step_nr, $cart ){
		$result = false;
		$generate = false;
		$coupon_enabled = $this->coupon_enabled( $recovery, $option, $step_nr );
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);

		if( !$coupon_enabled ){ //If coupon disabled, exit
			$this->update_cart_coupon( $code = false, $recovery, $step_nr, $cart ); //Remove coupon code from the user in case coupons are disabled
			return $result;
		}

		if( $coupon_enabled == 'generate_new' ){ //Generate new coupon code
			$generate = true;
			$code = $this->create_coupon( $recovery, $option, $step_nr, $cart );

		}else{ //Use existing coupon code
			$code = $admin->get_item_name_by_id( $coupon_enabled, 'shop_coupon' );
		}

		if($this->can_apply_coupon_code( $cart, $code )){ //If coupon code can be applied to given abandoned cart
			$this->update_cart_coupon( $code, $recovery, $step_nr, $cart ); //Save coupon code in the database accordingly to the recovery option that was used
			$generate_coupon = $this->get_coupon_settings( $recovery, $option, $step_nr, 'generate_coupon' );
			$result = array(
				'coupon_code'			=> $code,
				'generate'				=> $generate_coupon,
				'coupon_expiry'			=> $this->get_coupon_expiration_date( $code ),
				'coupon_description'	=> $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_description' )
			);

		}else{ //If coupon code was not valid for the current abandoned cart
			$this->update_cart_coupon( $code = false, $recovery, $step_nr, $cart ); //Remove coupon code from user in case coupon not applicable
			if($generate){ //If we previously generated coupon code - must delete it as the cart will not be able to apply it
				$coupon_id = $this->get_coupon_id( $code );
				wp_delete_post( $coupon_id, true );
			}
		}

		return $result;
	}

	/**
	* Returning coupon generation defaults
	*
	* @since    9.3
	* @return   array or string
	* @param    string     $value                Value to return
	*/
	public function get_defaults( $value = false ){
		$defaults = array(
			'discount_type'			=> 'percent',
			'coupon_amount'			=> 0,
			'coupon_shipping'		=> false,
			'coupon_expiry'			=> 10080, //One week
			'minimum_spend'			=> '',
			'minimum_spend_label'	=> esc_attr__('No minimum', 'woo-save-abandoned-carts'),
			'maximum_spend'			=> '',
			'maximum_spend_label'	=> esc_attr__('No maximum', 'woo-save-abandoned-carts'),
			'coupon_individual'		=> false,
			'coupon_sale'			=> false,
			'email_restriction'		=> 'any_email',
			'usage_limit_per_user'	=> apply_filters( 'cartbounty_pro_coupon_usage_limit_per_user', 0 ),
			'usage_limit'			=> apply_filters( 'cartbounty_pro_coupon_usage_limit', 1 ),
			'coupon_description'	=> esc_attr__( 'Please use the following coupon code to get a discount on your purchase.', 'woo-save-abandoned-carts' )
		);

		if($value){ //If a single value should be returned
			if(isset($defaults[$value])){ //Checking if value exists
				$defaults = $defaults[$value];
			}
		}

		return $defaults;
	}

	/**
	* Return coupon settings depending from the recovery type and step number
	*
	* @since    9.4
	* @return   array
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress etc.
	* @param    string     $option               Option data field name for storing values
	* @param    integer    $step_nr              Automation step number
	* @param    string     $value                Value to return
	*/
	public function get_coupon_settings( $recovery, $option, $step_nr, $value = false ){
		$settings = array();
		$saved_options = get_option($option);
		$coupon = (object)$saved_options;

		if( $recovery == 'wordpress' || $recovery == 'bulkgate' ){ //In case we are working with WordPress, must take into account that options are set in arrays
			$step_nr = str_replace(array( '[', ']' ), '', $step_nr); //Removing array brackets that have been added before
			$coupon = (object)$saved_options[$step_nr];
		}

		if( isset($coupon->generate_coupon) ){ //If coupon generation enabled
			$settings['generate_coupon'] = $coupon->generate_coupon;
		}else{
			$settings['generate_coupon'] = false;
		}

		if( !empty($coupon->existing_coupon) ){ //If existing coupon is set
			$settings['existing_coupon'] = $coupon->existing_coupon;
		}else{
			$settings['existing_coupon'] = '';
		}

		if( !empty($coupon->discount_type) ){ //If discount type is empty or is not set
			$settings['discount_type'] = $coupon->discount_type;
		}else{
			$settings['discount_type'] = $this->get_defaults( 'discount_type' );
		}

		if( !empty($coupon->coupon_amount) ){ //If custom amount is set
			$settings['coupon_amount'] = $coupon->coupon_amount;
		}else{
			$settings['coupon_amount'] = $this->get_defaults( 'coupon_amount' );
		}

		if( isset($coupon->coupon_shipping) ){ //If coupon free shiping enabled
			$settings['coupon_shipping'] = $coupon->coupon_shipping;
		}else{
			$settings['coupon_shipping'] = $this->get_defaults( 'coupon_shipping' );
		}

		$settings['coupon_expiry'] = $this->get_defaults( 'coupon_expiry' );
		if( !empty($coupon->coupon_expiry) ){ //If coupon expiry value is not empty
			$settings['coupon_expiry'] = $coupon->coupon_expiry;
		}elseif(isset($coupon->coupon_expiry) ){ //If the value is set
			if( $coupon->coupon_expiry == '0' ){
				$settings['coupon_expiry'] = $coupon->coupon_expiry;
			}
		}

		if( !empty($coupon->coupon_prefix) ){ //If custom prefix is set
			$settings['coupon_prefix'] = $coupon->coupon_prefix;
		}else{
			$settings['coupon_prefix'] = '';
		}

		if( !empty($coupon->minimum_spend) ){ //If minimum coupon spend is set
			$settings['minimum_spend'] = $coupon->minimum_spend;
		}else{
			$settings['minimum_spend'] = $this->get_defaults( 'minimum_spend' );
		}

		if( !empty($coupon->maximum_spend) ){ //If maximum coupon spend is set
			$settings['maximum_spend'] = $coupon->maximum_spend;
		}else{
			$settings['maximum_spend'] = $this->get_defaults( 'maximum_spend' );
		}

		if( isset($coupon->coupon_individual) ){ //If coupon free shiping enabled
			$settings['coupon_individual'] = $coupon->coupon_individual;
		}else{
			$settings['coupon_individual'] = $this->get_defaults( 'coupon_individual' );
		}

		if( isset($coupon->coupon_sale) ){ //If coupon free shiping enabled
			$settings['coupon_sale'] = $coupon->coupon_sale;
		}else{
			$settings['coupon_sale'] = $this->get_defaults( 'coupon_sale' );
		}

		if( !empty($coupon->coupon_include_products) ){ //If the selected products array is not empty
			$settings['coupon_include_products'] = $coupon->coupon_include_products;
		}else{
			$settings['coupon_include_products'] = array();
		}

		if( !empty($coupon->coupon_exclude_products) ){ //If the selected products array is not empty
			$settings['coupon_exclude_products'] = $coupon->coupon_exclude_products;
		}else{
			$settings['coupon_exclude_products'] = array();
		}

		if( !empty($coupon->coupon_include_categories) ){ //If the selected categories array is not empty
			$settings['coupon_include_categories'] = $coupon->coupon_include_categories;
		}else{
			$settings['coupon_include_categories'] = array();
		}

		if( !empty($coupon->coupon_exclude_categories) ){ //If the selected categories array is not empty
			$settings['coupon_exclude_categories'] = $coupon->coupon_exclude_categories;
		}else{
			$settings['coupon_exclude_categories'] = array();
		}

		if( !empty($coupon->email_restriction) ){ //If email_restrictions are not empty
			$settings['email_restriction'] = $coupon->email_restriction;
		}else{
			$settings['email_restriction'] = $this->get_defaults( 'email_restriction' );
		}

		if( !empty($coupon->coupon_description) ){ //If custom prefix is set
			$settings['coupon_description'] = $coupon->coupon_description;
		}else{
			$settings['coupon_description'] = '';
		}

		if($value){ //If a single value should be returned
			if(isset($settings[$value])){ //Checking if value exists
				$settings = $settings[$value];
			}
		}

		return $settings;
	}

	/**
	* Check if coupon enabled for the given recovery automation and step.
	* Returns dalse or a string "generate_new" or existing coupon code
	*
	* @since    9.4
	* @return   boolean or string
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    integer    $step_nr              Automation step number
	*/
	function coupon_enabled( $recovery, $option, $step_nr ){
		$enabled = false;
		$settings = (object)$this->get_coupon_settings( $recovery, $option, $step_nr );

		if( $settings->generate_coupon ){ //If coupon generation enabled
			$enabled = 'generate_new';
		}elseif( !empty($settings->existing_coupon) ){ //If existing coupon added, return existing coupon code
			$enabled = $settings->existing_coupon;
		}

		return $enabled;
	}

	/**
	* Create a new coupon. Returns coupon code value
	*
	* @since    9.4
	* @return   string
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    integer    $step_nr              Automation step number
	* @param    object     $cart                 Cart data
	*/
	private function create_coupon( $recovery, $option, $step_nr, $cart ){
		if(!$cart){ //If cart does not exist - exit and do not update as this is request is coming from email Preview
			return;
		}

		$coupon = new WC_Coupon();
		$allowed_emails = array();
		$coupon_code = $this->generate_coupon_code( $recovery, $option, $step_nr ); // Generate new coupon code
		$coupon_expiry = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_expiry' );
		if($coupon_expiry){ //If expiration date is set
			$coupon_expiry = date('Y-m-d', strtotime('+'. $coupon_expiry .' minutes')); //Converting selected time interval to a date
		}
		$email_restriction = $this->get_coupon_settings( $recovery, $option, $step_nr, 'email_restriction' );

		$coupon->set_discount_type( $this->get_coupon_settings( $recovery, $option, $step_nr, 'discount_type' ) ); //Set discount type
		$coupon->set_amount( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_amount' ) ); //Set coupon amount
		$coupon->set_free_shipping( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_shipping' ) ); //Set whether the discount awards a free shipping
		$coupon->set_date_expires( $coupon_expiry ); //Set discount expiry time
		$coupon->set_minimum_amount( $this->get_coupon_settings( $recovery, $option, $step_nr, 'minimum_spend' ) ); //Set minimum spend
		$coupon->set_maximum_amount( $this->get_coupon_settings( $recovery, $option, $step_nr, 'maximum_spend' ) ); //Set maximum spend
		$coupon->set_individual_use( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_individual' ) ); //Set wheather the coupon can be used in conjunction with other coupons
		$coupon->set_exclude_sale_items( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_sale' ) ); //Set if sale items are excluded from the discount or not
		$coupon->set_product_ids( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_include_products' ) ); //Allow just specific products with this discount
		$coupon->set_excluded_product_ids( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_exclude_products' ) ); //Exclude products from discount
		$coupon->set_product_categories( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_include_categories' ) ); //Set product categories included in the discount
		$coupon->set_excluded_product_categories( $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_exclude_categories' ) ); //Set product categories excluded from the discount

		if($email_restriction != 'any_email'){
			$allowed_emails = $cart->email;
		}

		$coupon->set_email_restrictions( $allowed_emails ); //Email address that can apply the coupon
		$coupon->set_code($coupon_code);
		$coupon->set_usage_limit_per_user( $this->get_defaults( 'usage_limit_per_user' ) ); //Set times discount can be used per user
		$coupon->set_usage_limit( $this->get_defaults( 'usage_limit' ) );
		$coupon->add_meta_data( 'cartbounty_pro_coupon', 1 ); //Adding additional meta data to easily find coupons created by CartBounty
		$coupon->save(); //Publish the coupon

		return $coupon_code;
	}

	/**
	* Generating unique coupon code
	*
	* @since    9.4
	* @return   string
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    integer    $step_nr              Automation step number
	* @param    string     $prefix               Use the prefix in case it is provided. Necessary in case of Email preview
	*/
	public function generate_coupon_code( $recovery, $option, $step_nr, $prefix = false ){
		$code = '';
		$coupon_prefix = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_prefix' );
		if($prefix || $prefix === ''){ //If prefix is set or it is empty
			$coupon_prefix = $prefix;
		}
		$length = apply_filters( 'cartbounty_pro_coupon_length', 12 );
		$allowed_characters = apply_filters( 'cartbounty_pro_coupon_characters', '23456789ABCEFGHJKLMNPRSTUVWXYZabcefghjkmnprstuvwxyz' );

		while ($length){ //Generate random coupon code
			$length = $length - 1;
			$code .= substr(str_shuffle($allowed_characters), 0, 1);
		}

		$code = implode( apply_filters( 'cartbounty_pro_coupon_separator', '_' ), str_split( $code, 5 ) ); //transform the code into a more readable and user friendly format
		return $coupon_prefix . $code;
	}

	/**
	* Get coupon ID from coupon code
	*
	* @since    9.4
	* @return   integer
	* @param    string     $coupon_code             Coupon code
	*/
	public function get_coupon_id( $coupon_code ){
		global $wpdb;
		$table = $wpdb->prefix . 'posts';
		$coupon_id = false;

		$coupon = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT id FROM {$table}
				WHERE post_title = %s AND
				post_type = %s",
				$coupon_code,
				'shop_coupon'
			)
		);

		if(isset($coupon->id)){
			$coupon_id = $coupon->id;
		}

		return $coupon_id;
	}

	/**
	* Get coupon code expiration date
	*
	* @since    9.4
	* @return   string
	* @param    string     $coupon_code             Coupon code
	*/
	public function get_coupon_expiration_date( $coupon_code ){
		if(!$coupon_code){
			return;
		}
		$date = '';
		$coupon = new WC_Coupon( $coupon_code );
		$date_expires = $coupon->get_date_expires();

		if($date_expires){ //If expiration date is set
			$date = $date_expires->date(apply_filters('cartbounty_pro_coupon_expiry_date_format', 'Y-m-d'));
		}
		
		return $date;
	}

	/**
	* Get coupon expiration time from minutes
	* This is ued to create expiration date in Preview
	*
	* @since    9.4
	* @return   string
	* @param    integer    $minutes             Minutes
	*/
	public function get_preview_coupon_expiration_date( $minutes ){
		if( !$minutes ){
			return;
		}
		$coupon_expiry = date(apply_filters('cartbounty_pro_coupon_expiry_date_format', 'Y-m-d'), strtotime('+'. $minutes .' minutes')); //Converting selected time interval to a date
		return $coupon_expiry;
	}

	/**
	* Check whether coupon code exists
	*
	* @since    9.4
	* @return   boolean
	* @param    string    $coupon_code       Coupon code that should be validated
	*/
	private function coupon_code_exists( $coupon_code = '' ){
		$exists = false;
		$coupon = new WC_Coupon( $coupon_code );
		if($coupon->is_valid()){
			$exists = true;
		}
		return $exists;
	}

	/**
	* Check if the cart is eligible for the coupon code and it can be applied
	*
	* @since    9.4
	* @return   boolean
	* @param    object    $cart              Cart data
	* @param    string    $coupon_code       Coupon code that should be validated
	*/
	private function can_apply_coupon_code( $cart, $coupon_code = '' ){
		if(!$cart){ //If cart does not exist - return true as this is request is coming from email Preview
			return true;
		}

		global $wpdb;
		$result = false;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$coupon = new WC_Coupon( $coupon_code );

		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$row = $wpdb->get_row( //Retrieve abandoned cart contents 
			$wpdb->prepare(
				"SELECT cart_contents
				FROM $cart_table
				WHERE id = %d AND
				(type = %d OR type = %d)",
				$cart->id,
				$admin->get_cart_type('abandoned'),
				$admin->get_cart_type('recovered_pending')
			)
		);

		wc_load_cart(); //Loading WooCommerce Cart object
		$admin->build_cart( $row, $ignore = true );
		$discounts = new WC_Discounts( WC()->cart );
		$response = $discounts->is_coupon_valid( $coupon );

		if ( !is_wp_error( $response ) ) { //If the coupon is valid
			$result = true;
		}else{
			if(!isset($response->errors['invalid_coupon'])){
				return;
			}
			$reason = implode('', $response->errors['invalid_coupon']);
			$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
			$admin->log('notice', sprintf(
				/* translators: %d - Email */
				esc_html__( 'Coupons: The cart ID = %d was not eligible for the coupon. Reason - %s', 'woo-save-abandoned-carts'), esc_html( $cart->id ), wp_strip_all_tags($reason) ));
		}

		WC()->session->__unset('cartbounty_pro_ignore_cart'); //Unsetting cart ignore cookie. If this is not done registered users or administrators may be unable to save carts after a coupon creation function is run
		
		return $result;
	}

	/**
	* Update coupon code of the abandoned cart
	*
	* @since    9.4
	* @param    string     $coupon_code       	 Coupon code that should be validated
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    integer    $step_nr              Automation step number
	* @param    object / string     $id          Cart data or Session ID
	*/
	public function update_cart_coupon( $coupon_code, $recovery, $step_nr, $id ){
		if( !$id ) return; //If cart does not exist - exit and do not update as this is request is coming from email Preview

		global $wpdb;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$identifier_name = 'session_id';
		$identifier_value = $id;
		$identifier_type = '%s';

		if( is_object( $id ) ){ //If identifier is Object - coupon update request is coming from CartBounty coupon generation
			$identifier_name = 'id';
			$identifier_value = $id->id;
			$identifier_type = '%d';
		}

		$row = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT coupons
				FROM $cart_table
				WHERE $identifier_name = $identifier_type",
				$identifier_value
			), ARRAY_A
		);

		$coupons = $row['coupons'];

		if( !empty( $coupons ) ){ //If the field already has data - unserialize it so we can use it as an array
			$coupons = @unserialize( $coupons );
		
		}else{
			$coupons = array();
		}

		if( $recovery == 'wordpress' || $recovery == 'bulkgate' ){ //In the case of WordPress or BulkGate recovery - we have additional key for the step
			
			if( !$coupon_code ){//If empty coupon code - remove element from array
				unset( $coupons[$recovery][$step_nr] );

			}else{
				$coupons[$recovery][$step_nr] = $coupon_code;
			}
			
		}else{

			if( !$coupon_code ){//If empty coupon code - remove element from array
				unset( $coupons[$recovery] );

			}else{
				$coupons[$recovery] = $coupon_code;
			}
		}

		$wpdb->query(
			$wpdb->prepare( "UPDATE $cart_table
				SET coupons = %s
				WHERE $identifier_name = $identifier_type AND
				(type = %d OR type = %d)",
				sanitize_text_field( serialize( $coupons ) ),
				$identifier_value,
				$admin->get_cart_type('abandoned'),
				$admin->get_cart_type('recovered_pending')
			)
		);
	}

	/**
	* Method displays available coupon discount types
	*
	* @since    9.4
	* @return   HTML
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    string     $step_nr              Automation step number
	* @param    string     $automation           Name for labeling
	* @param    string     $field_id             Field ID for distinguishing input fields
	*/
	public function display_coupon_types( $recovery, $option, $step_nr, $automation, $field_id ){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$selected_discount = $this->get_coupon_settings( $recovery, $option, $step_nr, 'discount_type' );

		$discount_types = array( //Defining array of discount types
			'percent'		=> esc_html__('Percentage discount', 'woo-save-abandoned-carts'),
			'fixed_cart'	=> esc_html__('Fixed cart discount', 'woo-save-abandoned-carts'),
			'fixed_product'	=> esc_html__('Fixed product discount', 'woo-save-abandoned-carts')
		);

		echo '<select id="cartbounty-pro-'. esc_attr( $automation ) .'-coupon-type'. esc_attr( $field_id ) .'" class="cartbounty-pro-select" name="'. esc_attr( $option ) . esc_attr( $step_nr ) .'[discount_type]" autocomplete="off" '. $admin->disable_field() .'>';
		foreach( $discount_types as $key => $discount ){
			echo "<option value='". esc_attr( $key ) ."' ". selected( $selected_discount, $key, false ) .">". esc_html( $discount ) ."</option>";
		}
		echo '</select>';
	}

	/**
	* Method displays available coupon discount types
	*
	* @since    9.4
	* @return   HTML
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    string     $step_nr              Automation step number
	* @param    string     $automation           Name for labeling
	* @param    string     $field_id             Field ID for distinguishing input fields
	*/
	public function display_coupon_expiry_options( $recovery, $option, $step_nr, $automation, $field_id ){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$selected_expiry = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_expiry' );
		$minutes = array( 1440, 2880, 4320, 5760, 7200, 8640, 10080, 20160, 30240, 40320, 0 ); //Defining array of minutes
		$zero_value = esc_html__( 'Never expires', 'woo-save-abandoned-carts' );
		$intervals = $admin->prepare_time_intervals( $minutes, $zero_value );

		echo '<select id="cartbounty-pro-'. esc_attr( $automation ) .'-coupon-expiry'. esc_attr( $field_id ) .'" class="cartbounty-pro-select" name="'. esc_attr( $option ) . esc_attr( $step_nr ) .'[coupon_expiry]" autocomplete="off" '. $admin->disable_field() .'>';
		foreach( $intervals as $key => $interval ){
			echo "<option value='". esc_attr( $key ) ."' ". selected( $selected_expiry, $key, false ) .">". esc_html( $interval ) ."</option>";
		}
		echo '</select>';
	}

	/**
	* Method displays available coupon email restriction options
	*
	* @since    9.4
	* @return   HTML
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    string     $step_nr              Automation step number
	* @param    string     $automation           Name for labeling
	* @param    string     $field_id             Field ID for distinguishing input fields
	*/
	public function display_coupon_email_restrictions( $recovery, $option, $step_nr, $automation, $field_id ){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$selected_restriction = $this->get_coupon_settings( $recovery, $option, $step_nr, 'email_restriction' );

		$restrictions = array( //Defining array of email options
			'any_email'			=> esc_html__('Any email', 'woo-save-abandoned-carts'),
			'abandoned_email'	=> esc_html__('Abandoned cart email', 'woo-save-abandoned-carts')
		);

		echo '<select id="cartbounty-pro-'. esc_attr( $automation ) .'-coupon-email-restriction'. esc_attr( $field_id ) .'" class="cartbounty-pro-select" name="'. esc_attr( $option ) . esc_attr( $step_nr ) .'[email_restriction]" autocomplete="off" '. $admin->disable_field() .'>';
		foreach( $restrictions as $key => $restriction ){
			echo "<option value='". esc_attr( $key ) ."' ". selected( $selected_restriction, $key, false ) .">". esc_attr( $restriction ) ."</option>";
		}
		echo '</select>';
	}

	/**
	* Returning HTML of coupon settings
	*
	* @since    9.4
	* @return   HTML
	* @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress, MailChimp
	* @param    string     $option               Option data field name for storing values
	* @param    integer    $step_nr              Automation step number
	*/
	public function display_coupon_settings( $recovery, $option, $step_nr = false ){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$automation = $recovery;
		$field_id = '';

		if( $recovery == 'wordpress' || $recovery == 'bulkgate' ){ //In case of of WordPress or BulkGate, must add additional items to output the form as we need to display coupons in all of the steps
			$automation = 'automation';
			$field_id = '-'.$step_nr;
			$step_nr = '['. $step_nr .']';
		}

		$generate_coupon = $this->get_coupon_settings( $recovery, $option, $step_nr, 'generate_coupon' );
		$existing_coupon = $this->get_coupon_settings( $recovery, $option, $step_nr, 'existing_coupon' );
		$coupon_amount = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_amount' );
		$coupon_shipping = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_shipping' );
		$coupon_prefix = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_prefix' );
		$minimum_spend = $this->get_coupon_settings( $recovery, $option, $step_nr, 'minimum_spend' );
		$maximum_spend = $this->get_coupon_settings( $recovery, $option, $step_nr, 'maximum_spend' );
		$coupon_individual = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_individual' );
		$coupon_sale = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_sale' );
		$coupon_include_products = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_include_products' );
		$coupon_exclude_products = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_exclude_products' );
		$coupon_include_categories = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_include_categories' );
		$coupon_description = $this->get_coupon_settings( $recovery, $option, $step_nr, 'coupon_description' );

		$find_items_nonce = wp_create_nonce( 'find_items' ); ?>

		<div class="cartbounty-pro-titles-column cartbounty-col-sm-12 cartbounty-col-md-4 cartbounty-col-lg-3">
			<h4><?php esc_html_e('Coupon', 'woo-save-abandoned-carts'); ?></h4>
			<p class="cartbounty-pro-titles-column-description">
				<?php esc_html_e('Add a coupon code to your recovery message as it might help to increase recovery ratio.', 'woo-save-abandoned-carts'); ?>
			</p>
		</div>
		<div class="cartbounty-pro-settings-column cartbounty-col-sm-12 cartbounty-col-md-8 cartbounty-col-lg-9<?php if($generate_coupon){ echo ' cartbounty-pro-checked-parent'; }?>">
			<div class="cartbounty-pro-settings-group cartbounty-pro-toggle">
				<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-generate-coupon<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-switch cartbounty-pro-control-visibility">
					<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-generate-coupon<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-checkbox" type="checkbox" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[generate_coupon]" value="1" <?php echo $admin->disable_field(); ?> <?php echo checked( 1, $generate_coupon, false ); ?> autocomplete="off" />
					<span class="cartbounty-pro-slider round"></span>
				</label>
				<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-generate-coupon<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-control-visibility"><?php esc_html_e('Generate coupon', 'woo-save-abandoned-carts'); ?></label>
			</div>
			<div class="cartbounty-pro-settings-group cartbounty-pro-hidden">
				<div class="cartbounty-pro-tabs">
					<ul class="cartbounty-pro-tabs-menu">
						<li>
							<button class="cartbounty-pro-tab-link active" id="cartbounty-pro-tab-general<?php echo esc_attr( $field_id ); ?>" data-cb-target="#cartbounty-pro-tab-general-content<?php echo esc_attr( $field_id ); ?>" type="button"><?php esc_html_e('General', 'woo-save-abandoned-carts'); ?></button>
						</li>
						<li>
							<button class="cartbounty-pro-tab-link" id="cartbounty-pro-tab-restrictions<?php echo esc_attr( $field_id ); ?>" data-cb-target="#cartbounty-pro-tab-restrictions-content<?php echo esc_attr( $field_id ); ?>" type="button"><?php esc_html_e('Usage restrictions', 'woo-save-abandoned-carts'); ?></button>
						</li>
					</ul>
					<div class="cartbounty-pro-tabs-contents">
						<div class="cartbounty-pro-tab active" id="cartbounty-pro-tab-general-content<?php echo esc_attr( $field_id ); ?>">
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-type<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Discount type', 'woo-save-abandoned-carts'); ?></label>
								<?php $this->display_coupon_types( $recovery, $option, $step_nr, $automation, $field_id ); ?>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-amount<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Coupon amount', 'woo-save-abandoned-carts'); ?></label>
								<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-amount<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-text" type="number" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_amount]" value="<?php echo esc_attr( $coupon_amount ); ?>" min="0" <?php echo $admin->disable_field(); ?> placeholder="<?php echo esc_attr( $this->get_defaults('coupon_amount') ); ?>" />
							</div>
							<div class="cartbounty-pro-settings-group cartbounty-pro-toggle">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-shipping<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-switch">
									<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-shipping<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-checkbox" type="checkbox" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_shipping]" value="1" <?php echo $admin->disable_field(); ?> <?php echo checked( 1, $coupon_shipping, false ); ?> autocomplete="off" />
									<span class="cartbounty-pro-slider round"></span>
								</label>
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-shipping<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Allow free shipping', 'woo-save-abandoned-carts'); ?></label>
								<p class='cartbounty-pro-additional-information'>
									<?php echo sprintf( wp_kses_post( __( 'Check this box if the coupon grants free shipping. A <a href="%s" target="_blank">free shipping method</a> must be enabled in your shipping zone and be set to require "a valid free shipping coupon" (see the "Free Shipping Requires" setting).', 'woo-save-abandoned-carts' ) ), 'https://docs.woocommerce.com/document/free-shipping/' ); ?>
								</p>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-expiry<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Coupon expires after', 'woo-save-abandoned-carts'); ?></label>
								<?php $this->display_coupon_expiry_options( $recovery, $option, $step_nr, $automation, $field_id ); ?>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-prefix<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Coupon prefix', 'woo-save-abandoned-carts'); ?></label>
								<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-prefix<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-text" type="text" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_prefix]" value="<?php echo esc_attr( stripslashes( $coupon_prefix ) ); ?>" <?php echo $admin->disable_field(); ?> />
								<p class='cartbounty-pro-additional-information'>
									<?php esc_html_e( 'You can use this to easily distinguish between generated coupons and track recovery results.', 'woo-save-abandoned-carts' ); ?>
								</p>
							</div>
						</div>
						<div class="cartbounty-pro-tab" id="cartbounty-pro-tab-restrictions-content<?php echo esc_attr( $field_id ); ?>">
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-minimum-spend<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Minimum spend', 'woo-save-abandoned-carts'); ?></label>
								<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-minimum-spend<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-text" type="number" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[minimum_spend]" value="<?php echo esc_attr( $minimum_spend ); ?>" min="0" <?php echo $admin->disable_field(); ?> placeholder="<?php echo esc_attr( $this->get_defaults('minimum_spend_label') ); ?>" />
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-maximum-spend<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Maximum spend', 'woo-save-abandoned-carts'); ?></label>
								<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-maximum-spend<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-text" type="number" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[maximum_spend]" value="<?php echo esc_attr( $maximum_spend ); ?>" min="0" <?php echo $admin->disable_field(); ?> placeholder="<?php echo esc_attr( $this->get_defaults('maximum_spend_label') ); ?>" />
							</div>
							<div class="cartbounty-pro-settings-group cartbounty-pro-toggle">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-individual<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-switch">
									<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-individual<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-checkbox" type="checkbox" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_individual]" value="1" <?php echo $admin->disable_field(); ?> <?php echo checked( 1, $coupon_individual, false ); ?> autocomplete="off" />
									<span class="cartbounty-pro-slider round"></span>
								</label>
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-individual<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Individual use only', 'woo-save-abandoned-carts'); ?></label>
								<p class='cartbounty-pro-additional-information'>
									<?php esc_html_e( 'Check this box if the coupon cannot be used in conjunction with other coupons.', 'woo-save-abandoned-carts' ); ?>
								</p>
							</div>
							<div class="cartbounty-pro-settings-group cartbounty-pro-toggle">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-sale<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-switch">
									<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-sale<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-checkbox" type="checkbox" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_sale]" value="1" <?php echo $admin->disable_field(); ?> <?php echo checked( 1, $coupon_sale, false ); ?> autocomplete="off" />
									<span class="cartbounty-pro-slider round"></span>
								</label>
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-sale<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Exclude sale items', 'woo-save-abandoned-carts'); ?></label>
								<p class='cartbounty-pro-additional-information'>
									<?php esc_html_e( 'Check this box if the coupon should not apply to items on sale. Per-item coupons will only work if the item is not on sale. Per-cart coupons will only work if there are items in the cart that are not on sale.', 'woo-save-abandoned-carts' ); ?>
								</p>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-include-products<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Products', 'woo-save-abandoned-carts'); ?></label>
								<select id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-include-products<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-select cartbounty-pro-multiple cartbounty-pro-ajax-search" multiple name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_include_products][]" placeholder="<?php esc_attr_e('Search products...', 'woo-save-abandoned-carts'); ?>" autocomplete="off" data-search-type="product" data-action="find_items" data-nonce="<?php echo esc_attr( $find_items_nonce ); ?>">
									<?php foreach( $coupon_include_products as $product ){
										echo "<option value='". esc_attr( $product ) ."' selected>". esc_html( $admin->get_item_name_by_id( $product, 'product' ) ) ."</option>";
									}?>
								</select>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-exclude-products<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Exclude products', 'woo-save-abandoned-carts'); ?></label>
								<select id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-exclude-products<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-select cartbounty-pro-multiple cartbounty-pro-ajax-search" multiple name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_exclude_products][]" placeholder="<?php esc_html_e('Search products...', 'woo-save-abandoned-carts'); ?>" autocomplete="off" data-search-type="product" data-action="find_items" data-nonce="<?php echo esc_attr( $find_items_nonce ); ?>">
									<?php foreach( $coupon_exclude_products as $product ){
										echo "<option value='". esc_attr( $product ) ."' selected>". esc_html( $admin->get_item_name_by_id( $product, 'product' ) ) ."</option>";
									}?>
								</select>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-include-product-categories<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Product categories', 'woo-save-abandoned-carts'); ?></label>
								<?php esc_html( $admin->display_product_categories( $recovery, $option, $step_nr, $automation, $field_id, 'include', $placement = 'coupons' ) ); ?>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-exclude-product-categories<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Exclude categories', 'woo-save-abandoned-carts'); ?></label>
								<?php esc_html( $admin->display_product_categories( $recovery, $option, $step_nr, $automation, $field_id, 'exclude', $placement = 'coupons' ) ); ?>
							</div>
							<div class="cartbounty-pro-settings-group">
								<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-email-restriction<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Coupon can be used by', 'woo-save-abandoned-carts'); ?></label>
								<?php esc_html( $this->display_coupon_email_restrictions( $recovery, $option, $step_nr, $automation, $field_id ) ); ?>
							</div>
						</div>
					</div>
				</div>
			</div>
			<div class="cartbounty-pro-settings-group cartbounty-pro-hidden-enabled">
				<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-existing-coupon<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Include an existing coupon', 'woo-save-abandoned-carts'); ?></label>
				<select id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-existing-coupon<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-select cartbounty-pro-ajax-search cartbounty-pro-hide-remove-button" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[existing_coupon]" placeholder="<?php esc_attr_e('Search coupon...', 'woo-save-abandoned-carts'); ?>" autocomplete="off" data-search-type="shop_coupon" data-action="find_items" data-nonce="<?php echo esc_attr( $find_items_nonce ); ?>">
					<?php if($existing_coupon) echo "<option value=". esc_attr( $existing_coupon ) ." selected>". esc_html( $admin->get_item_name_by_id( $existing_coupon, 'shop_coupon' ) ) ."</option>"; ?>
				</select>
			</div>
			<?php if($recovery == 'wordpress'): ?>
				<div class="cartbounty-pro-settings-group">
					<label for="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-description<?php echo esc_attr( $field_id ); ?>"><?php esc_html_e('Description', 'woo-save-abandoned-carts'); ?></label>
					<div class="cartbounty-pro-content-creation cartbounty-pro-flex">
						<input id="cartbounty-pro-<?php echo esc_attr( $automation ); ?>-coupon-description<?php echo esc_attr( $field_id ); ?>" class="cartbounty-pro-text" type="text" name="<?php echo esc_attr( $option ); ?><?php echo esc_attr( $step_nr ); ?>[coupon_description]" value="<?php echo esc_attr( stripslashes( $coupon_description ) ); ?>" placeholder="<?php echo esc_attr( $this->get_defaults('coupon_description') ); ?>" /><?php $admin->add_emojis(); ?><?php $admin->add_tags(); ?>
					</div>
				</div>
			<?php endif; ?>
		</div>
	<?php }

	/**
	* Delete expired coupons that have not been used and have been previously created by CartBounty
	* Coupons are erased in batches of 50 to avoid timeouts
	*
	* @since    9.4
	*/
	public function delete_unused_expired_coupons(){
		$args = array( //Selecting coupons that have expired, have been created by CartBounty and have not been used
			'posts_per_page'	=> 500,
			'post_type'			=> 'shop_coupon',
			'post_status'		=> 'publish',
			'meta_query'		=> array(
				'relation'		=> 'AND',
				array(
					'key'		=> 'date_expires',
					'value'		=> current_time( 'timestamp' ),
					'compare'	=> '<='
				),
				array(
					'key'		=> 'date_expires',
					'value'		=> '',
					'compare'	=> '!='
				),		
				array(
					'key'		=> 'cartbounty_pro_coupon',
					'value'		=> 1,
					'compare'	=> '='
				),
				array(
					'key'		=> 'usage_count',
					'value'		=> 0,
					'compare'	=> '='
				)
			),
			'orderby' => array( 
				'date_expires' => 'ASC'
			),
		);
		$coupons = get_posts( $args );

		if ( !empty($coupons) ) {
			foreach ($coupons as $coupon) {
				wp_delete_post( $coupon->ID, true );
			}
		}
	}
}