<?php
/**
 * The System status class.
 *
 * Used to define privacy related information
 *
 *
 * @since      9.2.1
 * @package    CartBounty Pro - Save and recover abandoned carts for WooCommerce
 * @subpackage CartBounty Pro - Save and recover abandoned carts for WooCommerce/includes
 * @author     Streamline.lv
 */
class CartBounty_Pro_System_Status{

	/**
	 * The ID of this plugin.
	 *
	 * @since    9.2.1
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    9.2.1
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    9.2.1
	 * @param    string    $plugin_name       The name of this plugin.
	 * @param    string    $version    		  The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ){
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Output system status information
	 *
	 * @since    9.2.1
	 * @return   HTML
	 */
	public function get_system_status(){

		if ( check_ajax_referer( 'get_system_status', 'nonce', false ) == false ) { //If the request does not include our nonce security check, stop executing the function
	        wp_send_json_error(esc_html__( 'Looks like you are not allowed to do this.', 'woo-save-abandoned-carts' ));
	    }

		global $wpdb;
		$carts = array();
		$all_plugins = array();
		$active_recovery = array();
		$exit_intent_options = array();
		$early_capture_options = array();
		$tab_notification_options = array();
		$settings = array();
		$missing_hooks = array();
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$activecampaign = new CartBounty_Pro_ActiveCampaign();
		$getresponse = new CartBounty_Pro_GetResponse();
		$mailchimp = new CartBounty_Pro_Mailchimp();
		$wordpress = new CartBounty_Pro_WordPress();
		$bulkgate = new CartBounty_Pro_BulkGate();
		$automation = new CartBounty_Pro_Automation();
		$webhook = new CartBounty_Pro_Webhook();
		$license = $admin->activate_license();
		$recaptcha = $admin->get_recaptcha();
		$overrides = $admin->get_template_overrides();
		
		if(get_option('cartbounty_pro_recoverable_cart_count')){
			$carts[] = esc_html__('Recoverable', 'woo-save-abandoned-carts' ) .': '. esc_html( get_option('cartbounty_pro_recoverable_cart_count') );
		}
		if(get_option('cartbounty_pro_ghost_cart_count')){
			$carts[] = esc_html__('Ghost', 'woo-save-abandoned-carts' ) .': '. esc_html( get_option('cartbounty_pro_ghost_cart_count') );
		}
		if(get_option('cartbounty_pro_recovered_cart_count')){
			$carts[] = esc_html__('Recovered', 'woo-save-abandoned-carts' ) .': '. esc_html( get_option('cartbounty_pro_recovered_cart_count') );
		}

		if($activecampaign->store_connected()){
			$active_recovery[] = 'ActiveCampaign';
			if(get_option('cartbounty_pro_activecampaign_automation_id')){
				$active_recovery[] = esc_html__('Automation', 'woo-save-abandoned-carts' ) .': '. esc_html( get_option('cartbounty_pro_activecampaign_automation_id') );
			}

			$excluded_settings = $this->get_excluded_data( 'activecampaign' );
			
			if( !empty( $excluded_settings ) ){
				$active_recovery[] = implode( ', ', $excluded_settings );
			}
		}
		if($getresponse->store_connected()){
			$active_recovery[] = 'GetResponse';

			$excluded_settings = $this->get_excluded_data( 'getresponse' );
			
			if( !empty( $excluded_settings ) ){
				$active_recovery[] = implode( ', ', $excluded_settings );
			}
		}
		if($mailchimp->store_connected()){
			$active_recovery[] = 'MailChimp';

			$excluded_settings = $this->get_excluded_data( 'mailchimp' );
			
			if( !empty( $excluded_settings ) ){
				$active_recovery[] = implode( ', ', $excluded_settings );
			}
		}
		if($wordpress->automation_enabled()){
			$active_recovery[] = 'WordPress';
			$active_recovery[] = esc_html__('Total emails sent', 'woo-save-abandoned-carts' ) .': '. esc_html( $automation->get_stats( 'wordpress', 'send' ) );
			$active_recovery[] = esc_html__('Total emails opened', 'woo-save-abandoned-carts' ) .': '. esc_html( $automation->get_stats( 'wordpress', 'open' ) );
			$active_recovery[] = esc_html__('Total emails clicked', 'woo-save-abandoned-carts' ) .': '. esc_html( $automation->get_stats( 'wordpress', 'click' ) );
			
			$excluded_settings = $this->get_excluded_data( 'wordpress' );
			
			if( !empty( $excluded_settings ) ){
				$active_recovery[] = implode( ', ', $excluded_settings );
			}
		}
		if($bulkgate->automation_enabled()){
			$bulkgate_settings = get_option( 'cartbounty_pro_bulkgate_settings' );
			$active_recovery[] = 'BulkGate';
			$active_recovery[] = esc_html__( 'Total messages sent', 'woo-save-abandoned-carts' ) .': '. esc_html( $automation->get_stats( 'bulkgate', 'send' ) );
			$active_recovery[] = esc_html__( 'Total messages clicked', 'woo-save-abandoned-carts' ) .': '. esc_html( $automation->get_stats( 'bulkgate', 'click' ) );
			if( $admin->tcpa_gdpr_confirmed( 'bulkgate' ) ){
				$active_recovery[] = esc_html__( 'TCPA / GDPR compliance', 'woo-save-abandoned-carts' ) .': '. esc_html__( 'Confirmed' );
			}
			if( isset( $bulkgate_settings[ 'enable_quiet_hours' ] ) ){ //If quiet hours enabled
				$active_recovery[] = esc_html__( 'Quiet hours', 'woo-save-abandoned-carts' ) .': '. esc_html__( 'Enabled' );
			}

			$excluded_settings = $this->get_excluded_data( 'bulkgate' );
			
			if( !empty( $excluded_settings ) ){
				$active_recovery[] = implode( ', ', $excluded_settings );
			}
		}
		if($webhook->webhook_connected()){
			$active_recovery[] = 'Webhook';

			$excluded_settings = $this->get_excluded_data( 'webhook' );
			
			if( !empty( $excluded_settings ) ){
				$active_recovery[] = implode( ', ', $excluded_settings );
			}
		}

		if(get_option('cartbounty_pro_exit_intent_status')){
			$exit_intent_options[] = esc_html__('Enabled', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_exit_intent_test_mode')){
			$exit_intent_options[] = esc_html__('Test mode', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_exit_intent_mobile_status')){
			$exit_intent_options[] = esc_html__('Mobile', 'woo-save-abandoned-carts' );
		}

		if(get_option('cartbounty_pro_early_capture_status')){
			$early_capture_options[] = esc_html__('Enabled', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_early_capture_test_mode')){
			$early_capture_options[] = esc_html__('Test mode', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_early_capture_mandatory')){
			$early_capture_options[] = esc_html__('Mandatory', 'woo-save-abandoned-carts' );
		}

		if(get_option('cartbounty_pro_tab_notification_status')){
			$tab_notification_options[] = esc_html__('Enabled', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_tab_notification_test_mode')){
			$tab_notification_options[] = esc_html__('Test mode', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_tab_notification_favicon_status')){
			$tab_notification_options[] = esc_html__('Favicon enabled', 'woo-save-abandoned-carts' );
		}

		if(get_option('cartbounty_pro_exclude_ghost_carts')){
			$settings[] = esc_html__('Exclude ghost carts', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_allowed_countries')){
			if(is_array(get_option('cartbounty_pro_allowed_countries'))){
				$countries = implode(", ", get_option('cartbounty_pro_allowed_countries'));
				$settings[] = esc_html__('Allowed countries', 'woo-save-abandoned-carts' ) .' ('. esc_html( $countries ) .')';
			}
		}
		if(get_option('cartbounty_pro_notification_frequency')){
			$frequency = get_option('cartbounty_pro_notification_frequency');
			if(isset($frequency['hours'])){
				$settings[] = esc_html__('Notification frequency', 'woo-save-abandoned-carts' ) .': '. esc_html( $frequency['hours'] );
			}
		}
		if(get_option('cartbounty_pro_notification_email')){
			$settings[] = esc_html__('Email', 'woo-save-abandoned-carts' ) .' ('. esc_html( get_option('cartbounty_pro_notification_email') ) .')';
		}
		if(isset($recaptcha['status'])){
			if($recaptcha['status']){
				$settings[] = esc_html__('ReCAPTCHA enabled', 'woo-save-abandoned-carts' );
			}
		}
		if( $admin->international_phone_enabled() ){
			$settings[] = esc_html__( 'Enable easy international phone input', 'woo-save-abandoned-carts' );
		}
		if( $admin->phone_consent_enabled() ){
			$settings[] = esc_html__( 'Enable phone number consent', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_lift_email')){
			$settings[] = esc_html__('Lift email field', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_lift_phone')){
			$settings[] = esc_html__('Lift phone field', 'woo-save-abandoned-carts' );
		}
		if(get_option('cartbounty_pro_logging_status')){
			$settings[] = esc_html__('Enable logging', 'woo-save-abandoned-carts' );
		}

		if(wp_next_scheduled('cartbounty_pro_notification_sendout_hook') === false){
			$missing_hooks[] = 'cartbounty_pro_notification_sendout_hook';
		}
		if(wp_next_scheduled('cartbounty_pro_sync_hook') === false){
			$missing_hooks[] = 'cartbounty_pro_sync_hook';
		}
		if(wp_next_scheduled('cartbounty_pro_remove_empty_carts_hook') === false) {
			$missing_hooks[] = 'cartbounty_pro_remove_empty_carts_hook';
		}

		$active_theme = '-';
		$active_theme_data = wp_get_theme();

		if( $active_theme_data ){
			$active_theme = $active_theme_data->get( 'Name' ) . ' ' . esc_html__( 'by', 'woo-save-abandoned-carts' ) . ' ' . $active_theme_data->get( 'Author' ) . ' ' . esc_html__( 'version', 'woo-save-abandoned-carts' ) . ' ' . $active_theme_data->get( 'Version' ) . ' (' . $active_theme_data->get( 'ThemeURI' ) . ')';
		}

		$active_plugins = (array) get_option( 'active_plugins', array() ); //Check for active plugins
		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
		}
		
		foreach ( $active_plugins as $plugin ) {
			$plugin_data    = @get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin );
			$dirname        = dirname( $plugin );

			if ( ! empty( $plugin_data['Name'] ) ) {
				// link the plugin name to the plugin url if available
				$plugin_name = $plugin_data['Name'];
				$all_plugins[] = $plugin_name . ' ' . esc_html__( 'by', 'woo-save-abandoned-carts' ) . ' ' . $plugin_data['Author'] . ' ' . esc_html__( 'version', 'woo-save-abandoned-carts' ) . ' ' . $plugin_data['Version'];
			}
		}

		$site_wide_plugins = '-';
		if ( sizeof( $all_plugins ) != 0 ) {
			$site_wide_plugins = implode( ', <br/>', $all_plugins );
		}

		$environment = array(
			esc_html__('WordPress address (URL)', 'woo-save-abandoned-carts' ) => home_url(),
			esc_html__('Site address (URL)', 'woo-save-abandoned-carts' ) => site_url(),
			esc_html__('WordPress version', 'woo-save-abandoned-carts' ) => get_bloginfo( 'version' ),
			esc_html__('WordPress multisite', 'woo-save-abandoned-carts' ) => (is_multisite()) ? esc_html__('Yes', 'woo-save-abandoned-carts' ) : '-',
			esc_html__('WooCommerce version', 'woo-save-abandoned-carts' ) => class_exists( 'WooCommerce' ) ? esc_html( WC_VERSION ) : '-',
			esc_html__('Server info', 'woo-save-abandoned-carts' ) => isset( $_SERVER['SERVER_SOFTWARE'] ) ? esc_html( $_SERVER['SERVER_SOFTWARE'] ) : '',
			esc_html__('PHP version', 'woo-save-abandoned-carts' ) => phpversion(),
			esc_html__('MySQL Version', 'woo-save-abandoned-carts' ) => $wpdb->db_version(),
			esc_html__('WordPress debug mode', 'woo-save-abandoned-carts' ) => ( defined( 'WP_DEBUG' ) && WP_DEBUG ) ? esc_html__('On', 'woo-save-abandoned-carts' ) : '-',
			esc_html__('WordPress cron', 'woo-save-abandoned-carts' ) => !( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON ) ? esc_html__('On', 'woo-save-abandoned-carts' ) : '-',
			esc_html__('Language', 'woo-save-abandoned-carts' ) => get_locale(),
			esc_html__('Default server timezone', 'woo-save-abandoned-carts' ) => date_default_timezone_get()
		);

		$cartbounty_settings = array(
			esc_html__('CartBounty Pro version', 'woo-save-abandoned-carts' ) => esc_html( $this->version ),
			esc_html__('License key', 'woo-save-abandoned-carts' ) => esc_html( $license['message'] ),
			esc_html__('Saved carts', 'woo-save-abandoned-carts' ) => ($carts) ? implode(", ", $carts) : '-',
			esc_html__('Recovery', 'woo-save-abandoned-carts' ) => ($active_recovery) ? implode(", ", $active_recovery) : '-',
			esc_html__('Exit Intent', 'woo-save-abandoned-carts' ) => ($exit_intent_options) ? implode(", ", $exit_intent_options) : '-',
			esc_html__('Early capture', 'woo-save-abandoned-carts' ) => ($early_capture_options) ? implode(", ", $early_capture_options) : '-',
			esc_html__('Tab notification', 'woo-save-abandoned-carts' ) => ($tab_notification_options) ? implode(", ", $tab_notification_options) : '-',
			esc_html__('Settings', 'woo-save-abandoned-carts' ) => ($settings) ? implode(", ", $settings) : '-',
			esc_html__('Missing hooks', 'woo-save-abandoned-carts' ) => ($missing_hooks) ? implode(", ", $missing_hooks) : '-',
			esc_html__('Template overrides', 'woo-save-abandoned-carts' ) => ($overrides) ? implode(", ", $overrides) : '-'
		);

		$output = '<table id="cartbounty-system-report-table">';
		$output .= '<tbody>
						<tr>
							<td class="section-title">###'. esc_html__( 'Environment', 'woo-save-abandoned-carts' ) .'###</td>
						</tr>';
					foreach( $environment as $key => $value ){
						$output .= '
						<tr>
							<td>'. esc_html( $key ) . ':' . '</td>
							<td>'. esc_html( $value ) .'</td>
						</tr>';
					}
		$output .= '</tbody>';
		$output .= '<tbody>
						<tr>
							<td class="section-title"></td>
						</tr>
						<tr>
							<td class="section-title">###'. esc_html__( CARTBOUNTY_PRO_ABREVIATION, 'woo-save-abandoned-carts' ) .'###</td>
						</tr>';
					foreach( $cartbounty_settings as $key => $value ){
						$output .= '
						<tr>
							<td>'. esc_html( $key ) . ':' . '</td>
							<td>'. esc_html( $value ) .'</td>
						</tr>';
					}
		$output .= '</tbody>';
		$output .= '<tbody>
						<tr>
							<td class="section-title"></td>
						</tr>
						<tr>
							<td class="section-title">###'. esc_html__( 'Themes', 'woo-save-abandoned-carts' ) .'###</td>
						</tr>
						<tr>
							<td>'. esc_html__('Active theme', 'woo-save-abandoned-carts' ) .':</td>
							<td>'. $active_theme .'</td>
						</tr>';
		$output .= '</tbody>';
		$output .= '<tbody>
						<tr>
							<td class="section-title"></td>
						</tr>
						<tr>
							<td class="section-title">###'. esc_html__( 'Plugins', 'woo-save-abandoned-carts' ) .'###</td>
						</tr>
						<tr>
							<td>'. esc_html__('Active plugins', 'woo-save-abandoned-carts' ) .':</td>
							<td>'. $site_wide_plugins .'</td>
						</tr>
					</tbody>';
		$output .= '</table>';

		wp_send_json_success( $output );
	}

	/**
	 * Return excluded data depending on recovery
	 *
	 * @since    9.7.5
	 * @return   array
	 * @param    string     $recovery             Recovery type e.g. ActiveCampaign, WordPress etc.
	 */
	public function get_excluded_data( $recovery ){
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$option = $admin->get_exclusion_option_name( $recovery );
		$exclusion_settings = $admin->get_exclusion_settings( $recovery, $option );
		$active_recovery = array();

		if( !empty( $exclusion_settings['enable_exclusions'] ) ){
			$active_recovery[] = esc_html__( 'Exclusions', 'woo-save-abandoned-carts' ) .': '. esc_html__( 'Enabled' );
			
			if( !empty( $exclusion_settings['excluded_products'] ) ){
				$active_recovery[] = esc_html__( 'Excluded products', 'woo-save-abandoned-carts' ) .': '. implode( '; ', $exclusion_settings['excluded_products'] );
			}

			if( !empty( $exclusion_settings['exclude_product_categories'] ) ){
				$active_recovery[] = esc_html__( 'Excluded categories', 'woo-save-abandoned-carts' ) .': '. implode( '; ', $exclusion_settings['exclude_product_categories'] );
			}
		}
		return $active_recovery;
	}

}