<?php
/**
 * The webhook class
 *
 * Used to define webhook related functions
 *
 *
 * @since      9.9
 * @package    CartBounty Pro - Save and recover abandoned carts for WooCommerce
 * @subpackage CartBounty Pro - Save and recover abandoned carts for WooCommerce/includes
 * @author     Streamline.lv
 */

class CartBounty_Pro_Webhook{

	/**
	 * Starting automated sending of abandoned carts to webhook 
	 *
	 * @since    9.9
	 */
	public function auto_send(){
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );

		if( !$admin->check_license() || !class_exists('WooCommerce') ){ //Exit if license key not valid or if WooCommerce is not activated
			return;

		}else{

			if( $this->webhook_connected() ){ //If webhook connected and enabled
				$this->put_carts();
			}
		}
	}

	/**
	 * Method retrieves saved webhook URL
	 *
	 * @since    9.9
	 * @return 	 String
	 */
	public function get_webhook_url(){
		$url = esc_attr( get_option( 'cartbounty_pro_webhook_url' ) ); //Retrieve GetResponse API key from database
		return $url;
	}

	/**
	 * Method returns array of webhook actions
	 *
	 * @since    9.9
	 * @return 	 Array
	 */
	public function get_webhook_actions(){
		$actions = array(
			'test_send_cart' 		=> esc_html( 'Add cart', 'woo-save-abandoned-carts' ),
			'test_update_cart' 		=> esc_html( 'Update cart', 'woo-save-abandoned-carts' ),
			'test_new_order' 		=> esc_html( 'New order', 'woo-save-abandoned-carts' ),
			'test_delete_cart' 		=> esc_html( 'Delete cart', 'woo-save-abandoned-carts' )
		);
		return $actions;
	}

	/**
	* Test the Webhook URL
	*
	* @since    9.9
	* @return 	Boolean
	*/
	public function credentials_test(){
		$result = $this->send_webhook( array(), 'GET' );
		return $result;
	}

	/**
	 * Method checks if webhook connection is valid
	 *
	 * @since    9.9
	 * @return 	 Boolean
	 */
	public function webhook_connected(){
		$status = false;
		$url = $this->get_webhook_url();

		if( !empty( $url ) && get_transient( 'cartbounty_pro_webhook_url' ) == $url ){ //If the URL is cahced, not changed, is valid and not empty
			$status = true;

		}elseif( !empty( $url ) ){ //If the URL is not empty

			if ( ( int )$this->credentials_test() ){ //In case if we get a valid response from webhook
				set_transient( 'cartbounty_pro_webhook_url', $url, 60*60*48 ); //Cache URL for 48 hours
				$status = true;
			}
		}

		return $status;
	}

	/**
	 * Method returns webhook status
	 *
	 * @since    9.9
	 * @return 	 Array
	 */
	public function api_status(){
		$url = $this->get_webhook_url();
		$response = array();

		if( !empty( $url ) ){ //If the field is not empty
			
			if( $this->webhook_connected() ){
				$response['status'] = '1';
				$response['result'] = '';
				$response['message'] =  esc_html__( 'Connected', 'woo-save-abandoned-carts' );
				set_transient('cartbounty_pro_webhook_url', $url, 60*60*48 ); //Cache URL for 48 hours

			}else{
				$response['status'] = '0';
				$response['result'] = esc_html__( 'Error', 'woo-save-abandoned-carts' );
				$response['message'] = esc_html__( 'Please make sure webhook URL is correct', 'woo-save-abandoned-carts' );
				delete_transient( 'cartbounty_pro_webhook_url' ); //Removing cached URL data
			}

		}else{ //In case the webhook URL field is empty
			$response['status'] = '0';
			$response['result'] = '';
			$response['message'] = '';
			delete_transient( 'cartbounty_pro_webhook_url' ); //Removing cached API Key data
		}

		return $response;
	}

	/**
	* Send Webhook
	* PUT 		- Method used to create new or update existing abandoned cart row
	* DELETE 	- Method used to delete abandoned cart
	*
	* @since    9.9
	* @param    array     $data    		  	  Data sent to webhook
	* @param    string    $method    		  Type of action (POST, GET, PUSH, PATCH, DELETE etc.)
	* @return 	Boolean
	*/
	private function send_webhook( $data, $method = 'PUT' ){
		$result = false;
		$url = $this->get_webhook_url();
		
		if( !empty( $data ) ){ //If data is not empty, encode it for sending over
			$data = json_encode( $data );
		}

		$request = wp_safe_remote_post( $url, array(
			'headers'     => [
				'Content-Type' => 'application/json',
			],
			'method'		=> $method,
			'timeout'		=> 45,
			'body'			=> $data
		));

		if ( is_wp_error( $request ) ) {
			$error = array();
			$error['code'] = $request->get_error_code();
			$error['message'] = $request->get_error_message();
			$this->log_wh_errors( $error );

		} else {

			if( isset( $request['response']['code'] ) ){

				if( $request['response']['code'] == 200 ){
					$this->log_wh_errors( $request['response'], $extra = array(
						'method' => $method
					) );
					$result = true;

				}else{

					if( isset( $request['response'] ) ){
						$this->log_wh_errors( $request['response'], $extra = array(
							'method' => $method
						) );
					}
				}
			}
		}

		return $result;
	}

	/**
	 * Function sends new or updates existing carts to webhook
	 *
	 * @since    9.9
	 */
	private function put_carts() {
		global $wpdb;
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$time = $admin->get_time_intervals();
		$ready_carts = array();

		$carts = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT *
				FROM {$cart_table}
				WHERE time != wh_last_synced AND
				(type = %d OR type = %d) AND
				(email != '' OR phone != '') AND
				cart_contents != '' AND
				paused != 1 AND
				wh_excluded != 1 AND
				time < %s AND
				time > %s
				ORDER BY time DESC",
				$admin->get_cart_type('abandoned'),
				$admin->get_cart_type('recovered_pending'),
				$time['wh_sync_period'],
				$time['maximum_sync_period']
			)
		);

		if( $carts ){ //If we have abandoned carts that must be created or updated
			foreach( $carts as $key => $cart ){

				if( !$admin->has_excluded_items( $cart, 'webhook' ) ){ //If cart contents do not have excluded items
					$ready_carts[] = $cart;

				}else{
					$admin->exclude_cart( 'webhook', $cart ); //Exclude cart from further recovery
				 	unset( $carts[$key] ); //Remove current cart from cart array since it includes products that are excluded
				}
			}

			$prepared_carts = $this->prepare_carts( $ready_carts );
			$result = $this->send_webhook( $prepared_carts );

			if ( $result ) { //Successful
				foreach( $ready_carts as $key => $cart ){
					$wpdb->query(
						$wpdb->prepare( 
							"UPDATE {$cart_table}
							SET wh_last_synced = %s
							WHERE id = %d",
							$cart->time,
							$cart->id
						)
					);
				}
				

			}else{ //Exclude the cart from further sending
				foreach( $ready_carts as $key => $cart ){
					$admin->exclude_cart( 'webhook', $cart ); //Exclude cart from further recovery
					$this->log_wh_errors( $result, $extra = array(
						'id' => $cart->id
					));
				}
			}
		}
	}

	/**
	 * Preparing cart data for sending
	 *
	 * @since    9.9
	 * @param    array     $carts    		  Abandoned carts
	 * @param    boolean   $test    		  Wheather this is a test or not
	 * @param    boolean   $order    		  Wheather this is a request for new order
	 * @return 	 Array
	 */
	public function prepare_carts( $carts, $test = false, $order = false ){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$coupons = new CartBounty_Pro_Coupons();
		$cart_data = array();

		if( !$test ){
			foreach ( $carts as $key => $cart ) {

				$coupon = array();
				$status = 'ordered';

				if( !$order ){
					$coupon = $coupons->get_coupon( 'webhook', 'cartbounty_pro_webhook_coupon', $step_nr = false, ( object )$cart ); //Coupon code creation
					$status = 'abandoned';
				}
				
				$checkout_url = $admin->create_cart_url( $cart->email, $cart->session_id, $cart->id, 'webhook' );
				$location_data = unserialize( $cart->location );
				$cart_content_lines = $this->prepare_lines( $cart->cart_contents ); //Creating lines from cart contents

				$cart_array = array(
					'id' 				=> $cart->id,
					'email' 			=> $cart->email,
					'phone' 			=> $cart->phone,
					'phone_consent' 	=> $cart->phone_consent,
					'name' 				=> $cart->name,
					'surname' 			=> $cart->surname,
					'country'			=> $location_data['country'],
					'city'				=> $location_data['city'],
					'postcode'			=> $location_data['postcode'],
					'language' 			=> $cart->language,
					'session_id' 		=> $cart->session_id,
					'cart_contents' 	=> $cart_content_lines,
					'cart_total'		=> $cart->cart_total,
					'currency'			=> $cart->currency,
					'coupon'			=> $coupon,
					'cart_url'			=> $checkout_url,
					'time'				=> $cart->time,
					'last_synced'		=> $cart->wh_last_synced,
					'cart_status'		=> $status,
					'store_name'		=> get_bloginfo( 'name' ),
					'store_url'			=> esc_url( get_bloginfo( 'url' ) )
				);

				$cart_data[] = ( object )$cart_array; //Turn cart array back into object and add it to array
			}

		}else{ //In case we are dealing with test data
			$cart_data = $carts;
		}

		return $cart_data;
	}

	/**
	 * Method prepares product lines
	 *
	 * @since    9.9
	 * @param    array       $cart_contents				Product array from abandoned cart
	 */
	private function prepare_lines( $cart_contents ){
		
		if( $cart_contents ){
			$line_id = 1;
			$lines = array();

			foreach( unserialize( $cart_contents ) as $cart_product ){ //Handling cart contents
				$product_id = $cart_product['product_id']; //Product ID
				$product_variant_id = $cart_product['product_variation_id']; //Product Variant ID
				$inventory_quantity = $cart_product['quantity']; //Quantity
				$product = wc_get_product( $product_id ); //Getting product data
				$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );

				if( $product ){ //If we have a product
					$title = $product->get_name();
					$url = get_permalink( $product->get_id() );
					$description = $product->get_short_description();
					$categories = strip_tags( wc_get_product_category_list( $product_id ) );
					$product_price = $admin->get_product_price( $cart_product );
					$image_url = $admin->get_product_thumbnail_url( $cart_product );

					if( empty( $product_variant_id ) ){
						$product_variant_id = $product_id; //Setting variation ID for simple products because it can't be null
					}

					$line = array(
						'name' 					=> $title,
						'price' 				=> $product_price,
						'quantity' 				=> $inventory_quantity,
						'product_id' 			=> $product_id,
						'product_variant_id' 	=> $product_variant_id,
						'categories' 			=> $categories,
						'description' 			=> $description,
						'product_url' 			=> $url,
						'image_url' 			=> $image_url
					);
					$lines[] = $line;
					$line_id++;

				}else{
					$admin->log( 'notice', sprintf(
						/* translators: %d - Product ID */
						esc_html__( 'Webhook: WooCommerce product does not exist. ID: %d', 'woo-save-abandoned-carts' ), esc_html( $product_id ) ) );
				}
			}

			return $lines;
		}
	}

	/**
	 * Method sends new order data to webhook
	 *
	 * @since    9.9
	 * @param    object     $cart            Abandoned cart that has just been turned into order
	 */
	public function sync_order( $cart ){

		if( !$this->webhook_connected() ) return;

		if( !$this->cart_synced( $cart->id ) ) return;

		$prepared_carts = $this->prepare_carts( array( $cart ), $test = false, $order = true );
		$result = $this->send_webhook( $prepared_carts );
	}

	/**
	 * Method sends abandoned cart delete request to webhook
	 *
	 * @since    9.9
	 * @return   Boolean
	 */
	public function delete_cart( $cart_id ){
		$deleted = false;
		$carts = array();

		if( !$this->webhook_connected() ) return;

		if( !is_array( $cart_id ) ){
			$cart_id = array( $cart_id );
		}

		foreach( $cart_id as $key => $id ){
			if( $this->cart_synced( $id ) ){
				$carts[] = array(
					'id' 				=> $id,
					'cart_status'		=> 'deleted',
					'store_name'		=> get_bloginfo( 'name' ),
					'store_url'			=> esc_url( get_bloginfo( 'url' ) )
				);
			}
		}

		if( !empty( $carts ) ){
			$result = $this->send_webhook( $carts, 'DELETE' );
			
			if( $result ){
				$deleted = true;
			}
		}

		return $deleted;
	}

	/**
	 * Method removes duplicate carts from webhook
	 *
	 * @since    9.5
	 * @param    array      $carts          Cart array
	 */
	public function delete_duplicate_carts( $carts ){

		if( !$this->webhook_connected() ) return;

		if( empty( $carts ) ) return; //Exit in case no carts provided

		$duplicate_carts = array();

		foreach( $carts as $key => $cart ) {
			if( $this->cart_synced( $cart->id ) ){
				$duplicate_carts[] = array(
					'id'				=> $cart->id,
					'cart_status'		=> 'deleted',
					'store_name'		=> get_bloginfo( 'name' ),
					'store_url'			=> esc_url( get_bloginfo( 'url' ) )
				);
			}
		}

		if( !empty( $duplicate_carts ) ){
			$result = $this->send_webhook( $duplicate_carts, 'DELETE' );
		}
	}

	/**
	 * Method returns information if abandoned cart has been sent to webhook
	 *
	 * @since    9.9
	 * @param    integer     $id - Abandoned cart ID
	 * @return 	 Bollean
	 */
	public function cart_synced( $id ){
		global $wpdb;
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$synced = false;

		$last_synced_time = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT wh_last_synced FROM $cart_table
				WHERE id = %d",
				intval( $id )
			)
		);

		if( $last_synced_time != '0000-00-00 00:00:00' ){ //If cart has been sent to webhook
			$synced = true;
		}

		return $synced;
	}

	/**
	* Test webhook data
	*
	* @since    9.9
	* @return   HTML
	*/
	public function test_webhook(){
		$direction = is_rtl() ? 'rtl' : 'ltr';

		if( !isset( $_POST ) ) wp_send_json_error();

		$nonce = $_POST['nonce'];
		$operation = $_POST['operation'];

		if ( check_ajax_referer( $operation, 'nonce', false ) == false ) { //If the request does not include our nonce security check, stop executing function
			wp_send_json_error('<span class="license-status license-inactive cartbounty-pro-response cartbounty-pro-no-border fadeOutSlow" dir="'. $direction .'"><i class="license-status-icon"><img src="'. esc_url( plugin_dir_url( __DIR__ ) ) . 'admin/assets/invalid-icon.svg" /></i>'. esc_html__( 'Looks like you are not allowed to do this.', 'woo-save-abandoned-carts' ) .'</span>' );
		}

		$test_data = $this->get_test_data( $operation );
		$carts = $test_data['carts'];
		$method = $test_data['method'];
		$prepared_carts = $this->prepare_carts( $carts, $test = true );
		$result = $this->send_webhook( $prepared_carts, $method );

		if( $result ){
			wp_send_json_success('<span class="license-status license-active cartbounty-pro-response cartbounty-pro-no-border fadeOutSlow" dir="'. $direction .'"><i class="license-status-icon"><img src="'. esc_url( plugin_dir_url( __DIR__ ) ) . 'admin/assets/active-icon.svg" /></i>'. esc_html__( 'Webhook successfully sent', 'woo-save-abandoned-carts' ) .'</span>' );
		}else{
			wp_send_json_error('<span class="license-status license-inactive cartbounty-pro-response cartbounty-pro-no-border fadeOutSlow" dir="'. $direction .'"><i class="license-status-icon"><img src="'. esc_url( plugin_dir_url( __DIR__ ) ) . 'admin/assets/invalid-icon.svg" /></i>'. esc_html__( 'Error sending test webhook.', 'woo-save-abandoned-carts' ) .'</span>' );
		}
	}

	/**
     * Returning webhook test data and method
     *
     * @since    9.9
     * @param    string     $action    		      Webhook action that needs to be performed
     * @return   array
     */
	public function get_test_data( $action ){
		$public = new CartBounty_Pro_Public( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$carts = array();
		$cart = array();
		$method = '';

		if( $action == 'test_send_cart' || $action == 'test_update_cart' || $action == 'test_new_order' ){
			$type = 'abandoned';
			$method = 'PUT';
			$last_synced = '0000-00-00 00:00:00';
			$current_time = current_time( 'mysql', false );

			if( $action == 'test_new_order' ){
				$type = 'ordered';
			}

			if( $action == 'test_update_cart' ){
				$last_synced = $current_time;
			}

			$product_a = array(
				'name' => esc_html__( 'Product name', 'woo-save-abandoned-carts' ),
				'price' => 28.99,
				'quantity' => 1,
				'product_id' => 750,
				'product_variant_id' => 753,
				'categories' => esc_html__( 'Category A, Category B', 'woo-save-abandoned-carts' ),
				'description' => esc_html__( 'Product description', 'woo-save-abandoned-carts' ),
				'product_url' => esc_url( get_bloginfo( 'url' ) ),
				'image_url' => esc_url( $public->get_plugin_url() . 'public/assets/email-product-image-1.png' )
			);

			$product_b = array(
				'name' => esc_html__( 'Product name', 'woo-save-abandoned-carts' ),
				'price' => 73,
				'quantity' => 2,
				'product_id' => 853,
				'product_variant_id' => 853,
				'categories' => '',
				'description' => esc_html__( 'Product description', 'woo-save-abandoned-carts' ),
				'product_url' => esc_url( get_bloginfo( 'url' ) ),
				'image_url' => esc_url( $public->get_plugin_url() . 'public/assets/email-product-image-2.png' )
			);

			$cart = array(
				'id' 				=> 1,
				'email' 			=> get_option( 'admin_email' ),
				'phone' 			=> esc_html__( 'Phone number', 'woo-save-abandoned-carts' ),
				'phone_consent' 	=> 0,
				'name' 				=> esc_html__( 'Name', 'woo-save-abandoned-carts' ),
				'surname' 			=> esc_html__( 'Surname', 'woo-save-abandoned-carts' ),
				'country'			=> 'US',
				'city'				=> esc_html__( 'City', 'woo-save-abandoned-carts' ),
				'postcode'			=> esc_html__( 'Postcode', 'woo-save-abandoned-carts' ),
				'language' 			=> 'en_US',
				'session_id' 		=> esc_html__( 'Session ID', 'woo-save-abandoned-carts' ),
				'cart_contents' 	=> array( $product_a, $product_b ),
				'cart_total'		=> 101.99,
				'currency'			=> 'USD',
				'coupon'			=> array(
					'coupon_code' 			=> esc_html__( 'Coupon code', 'woo-save-abandoned-carts' ),
					'coupon_expiry' 		=> date( 'M d, Y', strtotime( $current_time . ' + 14 days' ) ),
					'coupon_description' 	=> esc_html__( 'Coupon description', 'woo-save-abandoned-carts' )
				),
				'cart_url'			=> esc_url( get_bloginfo( 'url' ) ),
				'time'				=> $current_time,
				'last_synced'		=> $last_synced,
				'cart_status'		=> $type,
				'store_name'		=> get_bloginfo( 'name' ),
				'store_url'			=> esc_url( get_bloginfo( 'url' ) )
			);

		}elseif( $action == 'test_delete_cart' ){
			$type = 'deleted';
			$method = 'DELETE';

			$cart = array(
				'id' 				=> 1,
				'cart_status'		=> $type,
				'store_name'		=> get_bloginfo( 'name' ),
				'store_url'			=> esc_url( get_bloginfo( 'url' ) )
			);
		}

		$carts[] = ( object )$cart;

		return array(
			'carts'		=> $carts,
			'method'	=> $method
		);
	}

	/**
	* Function handles webhook error logging
	*
	* @since    9.9
	* @param    array     $error    		  Webhook error
	* @param    array     $extra    		  Additional information to output, optional
	*/
	private function log_wh_errors( $error, $extra = array() ){
		$additional_data = array();

		if( isset( $extra['id'] ) ){
			$additional_data[] = 'Cart: ' . $extra['id'];
		}

		if( isset( $extra['method'] ) ){
			$additional_data[] = 'Method: ' . $extra['method'];
		}

		if( isset( $error['code']) && $error['message'] ){
			$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
			$admin->log( 'notice', sprintf( 'Webhook: %s - %s. %s', esc_html( $error['code'] ), esc_html( $error['message'] ), esc_html( implode( ', ', $additional_data ) ) ) );
		}
	}
}