<?php

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and hooks to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    CartBounty Pro - Save and recover abandoned carts for WooCommerce
 * @subpackage CartBounty Pro - Save and recover abandoned carts for WooCommerce/public
 * @author     Streamline.lv
 */
 
class CartBounty_Pro_Public{
	
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0
	 * @param    string    $plugin_name       The name of the plugin.
	 * @param    string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		global $wpdb;
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the public area.
	 *
	 * @since    4.0
	 */
	public function enqueue_styles(){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		if( $this->tool_enabled( 'exit_intent' ) || $this->tool_enabled( 'early_capture' ) || $admin->international_phone_enabled() ){ //If Exit Intent or Early capture enabled
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/cartbounty-pro-public.css', array(), $this->version, 'all' );
			wp_style_add_data( $this->plugin_name, 'rtl', 'replace' );
		}
	}

	/**
	 * Register the javascripts for the public area.
	 *
	 * @since    4.0
	 */
	public function enqueue_scripts(){
		if (!class_exists('WooCommerce')) { //If WooCommerce does not exist
			return;
		}

		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);

		if( $this->tool_enabled( 'exit_intent' ) || $this->tool_enabled( 'early_capture' ) ){ //If Exit Intent or Early capture enabled, check if Google reCAPTCHA should be added
			//Setting default values
			$recaptcha_enabled = false;
			$recaptcha_site_key = false;

			//Logged in users do not have to go through reCAPTCHA validation
			if(is_user_logged_in()){
				$user_logged_in = true;
			}else{
				$user_logged_in = false;

				$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
				$recaptcha = $admin->get_recaptcha();

				if($recaptcha['enabled']){
					$recaptcha_enabled = $recaptcha['enabled'];
					$recaptcha_site_key = $recaptcha['site_key'];
					wp_enqueue_script( $this->plugin_name . '-recaptchav3', 'https://www.google.com/recaptcha/api.js?render=' . $recaptcha_site_key, false, $this->version, false ); //Add reCAPTCHA v3 script on checkout form
				}
			}
		}

		if( $admin->international_phone_enabled() ){ //If international phone number input enabled
			$data = array(
				'preferredCountries'    => $admin->get_international_phone_preferred_countries(),
				'initialCountry' 		=> $admin->get_default_country(),
				'separateDialCode' 		=> false,
				'utilsScript'           => plugin_dir_url( __FILE__ ) . 'js/utils.js',
				'ajaxurl' 				=> admin_url( 'admin-ajax.php' )
			);

			wp_enqueue_script( $this->plugin_name . '-international-phone-input', plugin_dir_url( __FILE__ ) . 'js/cartbounty-pro-public-phone-input.js', array( 'jquery' ), $this->version, true );
			wp_localize_script( $this->plugin_name . '-international-phone-input', 'cartbounty_ip', $data ); //Sending variable over to JS file
		}

		if($this->tool_enabled( 'exit_intent' )){ //If Exit Intent Enabled
			$hours = esc_html(apply_filters( 'cartbounty_pro_exit_intent_reset_hours', 1));
			$mobile_exit_intent_enabled = get_option('cartbounty_pro_exit_intent_mobile_status');
			if(get_option('cartbounty_pro_exit_intent_test_mode')){ //If test mode is enabled
				$hours = 0;
				$mobile_exit_intent_enabled = true;
			}
			$language = get_locale();

			$data = array(
				'hours' => $hours,
				'product_count' => $this->get_cart_product_count(),
				'is_user_logged_in' => $user_logged_in,
				'recaptcha_enabled' => $recaptcha_enabled,
				'recaptcha_site_key' => $recaptcha_site_key,
				'mobile_exit_intent_enabled' => $mobile_exit_intent_enabled,
				'phone_validation' => apply_filters( 'cartbounty_pro_phone_validation', '^[+0-9\s]\s?\d[0-9\s-.]{6,30}$'),
				'language' => $language,
				'ajaxurl' => admin_url( 'admin-ajax.php' )
			);
			wp_enqueue_script( $this->plugin_name . '-exit-intent', plugin_dir_url( __FILE__ ) . 'js/cartbounty-pro-public-exit-intent.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( $this->plugin_name . '-exit-intent', 'cartbounty_ei', $data); //Sending variable over to JS file
		}
		
		if($this->tool_enabled( 'early_capture' )){ //If Early capture is enabled
			$cart_recoverable = $this->cart_recoverable();
			if( $cart_recoverable && !get_option('cartbounty_pro_early_capture_test_mode') ){ //If cart has already been saved and it is recoverable and Test mode is disabled, we do not need to include Early capture
				return;
			}
			$hours = esc_html(apply_filters( 'cartbounty_pro_early_capture_reset_hours', 1));
			$mandatory_input = false;
			if(get_option('cartbounty_pro_early_capture_test_mode')){ //If test mode is enabled
				$hours = 0;
			}
			if(get_option('cartbounty_pro_early_capture_mandatory')){ //If mandatory input enabled
				$mandatory_input = true;
			}
			$style = esc_attr( get_option('cartbounty_pro_early_capture_type'));
			$language = get_locale();

			$data = array(
			    'hours' => $hours,
			    'is_user_logged_in' => $user_logged_in,
		   		'recaptcha_enabled' => $recaptcha_enabled,
		    	'recaptcha_site_key' => $recaptcha_site_key,
			    'mandatory_input' => $mandatory_input,
			    'phone_validation' => apply_filters( 'cartbounty_pro_phone_validation', '^[+0-9\s]\s?\d[0-9\s-.]{6,30}$'),
			    'style' => $style,
			    'language' => $language,
			    'ajaxurl' => admin_url( 'admin-ajax.php' )
			);
			if($style !=2){ //In case we must display Early capture as a popup, we do not require tooltipster
				wp_enqueue_script( $this->plugin_name . '-tooltipster', plugin_dir_url( __FILE__ ) . 'js/tooltipster.bundle.min.js', array( 'jquery' ), $this->version, false );
			}

			wp_enqueue_script( $this->plugin_name . '-early-capture', plugin_dir_url( __FILE__ ) . 'js/cartbounty-pro-public-early-capture.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( $this->plugin_name . '-early-capture', 'cartbounty_ec', $data); //Sending variable over to JS file
		}

		if( $this->tool_enabled( 'tab_notification', $ignore_logged_users = true ) ){ //If Tab notification is enabled
			$message = $admin->get_tab_notification_message();
			$check_cart = false;
			$interval = array(
				'miliseconds' => $admin->get_tools_defaults( 'interval', 'tab_notification' )
			);

			if( get_option( 'cartbounty_pro_tab_notification_check_cart' ) ){ //If test mode is enabled
				$check_cart = true;
			}

			if( get_option( 'cartbounty_pro_tab_notification_interval' ) ){
				$interval = get_option( 'cartbounty_pro_tab_notification_interval' );
			}

			//Prepare Favicon
			$image_id = get_option( 'cartbounty_pro_tab_notification_favicon' );
			$image_url = $this->get_plugin_url() . '/public/assets/tab-notification-favicon.png';
			
			if( $image_id ){
				$image = wp_get_attachment_image_src( $image_id, 'full' );
				if( is_array( $image ) ){
					$image_url = $image[0];
				}
			}

			$data = array(
				'product_count' => $this->get_cart_product_count(),
				'message' => $message['value'],
				'check_cart' => $check_cart,
				'interval' => apply_filters( 'cartbounty_pro_tab_notification_interval', $interval['miliseconds'] ),
				'favicon_enabled' => get_option( 'cartbounty_pro_tab_notification_favicon_status' ),
				'favicon' => apply_filters( 'cartbounty_pro_tab_notification_favicon', $image_url ),
				'favicon_relationship' => apply_filters( 'cartbounty_pro_tab_notification_favicon_relationship', 'icon, shortcut icon' ),
				'ajaxurl' => admin_url( 'admin-ajax.php' )
			);

			wp_enqueue_script( $this->plugin_name . '-tab-notification', plugin_dir_url( __FILE__ ) . 'js/cartbounty-pro-tab-notification.js', array( 'jquery' ), $this->version, true );
			wp_localize_script( $this->plugin_name . '-tab-notification', 'cartbounty_tn', $data );
		}
	}

	/**
	 * Method to add aditional JS file to the checkout field and read data from inputs
	 *
	 * @since    1.0
	 */
	function add_additional_scripts_on_checkout(){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		if(!$admin->check_license()){//Exit if license key not valid
			return;
		}else{

			//Setting default values
			$recaptcha_enabled = false;
			$recaptcha_site_key = false;

			//Logged in users do not have to go through reCAPTCHA validation
			if(is_user_logged_in()){
				$user_logged_in = true;
			}else{
				$user_logged_in = false;

				$recaptcha = $admin->get_recaptcha();

				if($recaptcha['enabled']){
					$recaptcha_enabled = $recaptcha['enabled'];
					$recaptcha_site_key = $recaptcha['site_key'];
					wp_enqueue_script( $this->plugin_name . '-recaptchav3', 'https://www.google.com/recaptcha/api.js?render=' . $recaptcha_site_key, false, $this->version, false ); //Add reCAPTCHA v3 script on checkout form
				}
			}
			$language = get_locale();

			$data = array(
				'is_user_logged_in' => $user_logged_in,
			    'recaptcha_enabled' => $recaptcha_enabled,
			    'recaptcha_site_key' => $recaptcha_site_key,
			    'language' => $language,
			    'ajaxurl' => admin_url( 'admin-ajax.php' )
			);

			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/cartbounty-pro-public.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( $this->plugin_name, 'cartbounty_co', $data );
		}
	}
	
	/**
	 * Method that takes care of saving and updating carts
	 *
	 * @since    8.1
	 */
	function save_cart(){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		if(!$admin->check_license()){//Exit if license key not valid
			return;
		}
		if ( isset( $_GET['cartbounty-pro'] ) || isset( $_GET['cb'] ) ){ //Do not create or update cart in case we have a user who returns from abandoned cart reminder message
			return;
		}
		if(!WC()->cart){ //Exit if Woocommerce cart has not been initialized
			return;
		}
		$ghost = true;

		if(!$this->check_recaptcha()){ //Checking against bots
			return;
		}

		if(WC()->session->get('cartbounty_pro_ignore_cart')){ //Exit and do not save cart if the cart is created for coupon code validation prior to sending it out
			return;
		}

		if($this->cart_recoverable()){ //If cart is recoverable
			$ghost = false;
			$this->update_logged_customer_id(); //If current user had an abandoned cart before - restore session ID (in case of user switching)
		}

		if($this->exclude_ghost_cart() && $ghost){ //If Ghost carts are disabled and current cart is a ghost cart and current country should be excluded - do not save it
			return;
		}

		$cart = $this->read_cart();
		$cart_saved = $this->cart_saved($cart['session_id']);

		if( $cart_saved ){ //If cart has already been saved
			$this->update_cart( $cart, $ghost );

		}else{
			$this->create_new_cart( $cart, $ghost );
		}
	}

	/**
	 * Method creates a new cart
	 *
	 * @since    8.1
	 * @param    array      $cart     Cart contents
	 * @param    boolean    $ghost    If the cart is a ghost cart or not
	 */
	function create_new_cart( $cart = array(), $ghost = false ){
		global $wpdb;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$user_data = $this->get_user_data();

		//In case if the cart has no items in it, we must delete the cart
		if(empty( $cart['product_array'] )){
			$admin->clear_cart_data();
			return;
		}

		if($ghost){ //If dealing with a ghost cart
			//Inserting row into database
			$wpdb->query(
				$wpdb->prepare(
					"INSERT INTO $cart_table
					( location, cart_contents, cart_total, currency, time, session_id )
					VALUES ( %s, %s, %0.2f, %s, %s, %s )",
					array(
						'location'		=> sanitize_text_field( serialize( $user_data['location'] ) ),
						'products'		=> serialize( $cart['product_array'] ),
						'total'			=> sanitize_text_field( $cart['cart_total'] ),
						'currency'		=> sanitize_text_field( $cart['cart_currency'] ),
						'time'			=> sanitize_text_field( $cart['current_time'] ),
						'session_id'	=> sanitize_text_field( $cart['session_id'] )
					)
				)
			);
			$this->increase_ghost_cart_count();

		}else{
			$other_fields = NULL;
			$cart_source = $this->get_cart_source();
			$source = $cart_source['source'];
			$phone_consent = 0;

			if(empty($source)){ //If source not coming from Tools
				$source = 'NULL';

			}else{ //If source coming from Tools, phone consent is enabled, phone provided and is not empty - save phone consent
				if($cart_source['first']){ //If cart source saved for the first time
					if($admin->phone_consent_enabled()){
						if(isset($_POST['cartbounty_pro_phone'])){
							if(!empty($_POST['cartbounty_pro_phone'])){ //If phone number is not empty
								$phone_consent = 1;
							}
						}
					}
				}
			}

			if(!empty($user_data['other_fields'])){
				$other_fields = sanitize_text_field( serialize( $user_data['other_fields'] ) );
			}

			//Inserting row into Database
			$wpdb->query(
				$wpdb->prepare(
					"INSERT INTO $cart_table
					( name, surname, email, phone, phone_consent, location, cart_contents, cart_total, currency, time, session_id, language, other_fields, saved_via)
					VALUES ( %s, %s, %s, %s, %d, %s, %s, %0.2f, %s, %s, %s, %s, %s, %s)",
					array(
						'name'			=> sanitize_text_field( $user_data['name'] ),
						'surname'		=> sanitize_text_field( $user_data['surname'] ),
						'email'			=> sanitize_email( $user_data['email'] ),
						'phone'			=> filter_var( $user_data['phone'], FILTER_SANITIZE_NUMBER_INT),
						'phone_consent'	=> sanitize_text_field( $phone_consent ),
						'location'		=> sanitize_text_field( serialize( $user_data['location'] ) ),
						'products'		=> serialize( $cart['product_array'] ),
						'total'			=> sanitize_text_field( $cart['cart_total'] ),
						'currency'		=> sanitize_text_field( $cart['cart_currency'] ),
						'time'			=> sanitize_text_field( $cart['current_time'] ),
						'session_id'	=> sanitize_text_field( $cart['session_id'] ),
						'language'		=> sanitize_text_field( $user_data['language'] ),
						'other_fields'	=> $other_fields,
						'saved_via'		=> $source
					)
				)
			);
			$this->increase_recoverable_cart_count();
		}
		$this->set_cartbounty_session($cart['session_id']);
	}

	/**
	 * Method updates a cart
	 *
	 * @since    8.1
	 * @param    array      $cart     Cart contents
	 * @param    boolean    $ghost    If the cart is a ghost cart or not
	 */
	function update_cart( $cart = array(), $ghost = false ){
		$user_data = $this->get_user_data();

		//In case if the cart has no items in it, we must delete the cart
		if(empty( $cart['product_array'] )){
			$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
			$admin->clear_cart_data();
			return;
		}

		if($ghost){ //In case of a ghost cart
			$this->update_cart_data($cart);

		}else{
			$mailchimp = new CartBounty_Pro_Mailchimp();
			$mailchimp->delete_updated_cart( $cart ); //Sending a Delete request of the current cart over to MailChimp

			if(isset($_POST["action"])){
				if($_POST["action"] == 'cartbounty_pro_save'){ //In case update is coming from input form, update both cart and user data
					$this->update_cart_and_user_data($cart, $user_data);
				}else{
					$this->update_cart_data($cart);
				}
			}else{
				$this->update_cart_data($cart);
			}
		}
		
		$this->set_cartbounty_session($cart['session_id']);
	}

	/**
	 * Method updates only cart related data excluding customer details
	 *
	 * @since    8.1
	 * @param    Array    $cart    Cart contents inclding cart session ID
	 */
	function update_cart_data($cart){
		global $wpdb;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;

		$updated_rows = $wpdb->query(
			$wpdb->prepare(
				"UPDATE $cart_table
				SET cart_contents = %s,
				cart_total = %0.2f,
				currency = %s,
				time = %s
				WHERE session_id = %s AND
				(type = %d OR type = %d)",
				serialize( $cart['product_array'] ),
				sanitize_text_field( $cart['cart_total'] ),
				sanitize_text_field( $cart['cart_currency'] ),
				sanitize_text_field( $cart['current_time'] ),
				$cart['session_id'],
				$admin->get_cart_type('abandoned'),
				$admin->get_cart_type('recovered_pending')
			)
		);

		$this->delete_duplicate_carts( $cart['session_id'], $updated_rows);
	}

	/**
	 * Method updates both customer data and contents of the cart
	 *
	 * @since    8.1
	 * @param    Array    $cart    		Cart contents inclding cart session ID
	 * @param    Array    $user_data    User's data
	 */
	function update_cart_and_user_data($cart, $user_data){
		global $wpdb;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$other_fields = NULL;
		$cart_source = $this->get_cart_source();
		$source = $cart_source['source'];
		$phone_consent = $user_data['phone_consent'];

		if(empty($source)){ //If source not coming from Tools
			$source = 'NULL';

		}else{ //If source coming from Tools, phone consent is enabled, phone provided and is not empty - save phone consent
			if($cart_source['first']){ //If cart source saved for the first time
				if($admin->phone_consent_enabled()){
					if(isset($_POST['cartbounty_pro_phone'])){
						if(!empty($_POST['cartbounty_pro_phone'])){ //If phone number is not empty
							$phone_consent = 1;
						}
					}
				}
			}
		}

		if(!empty($user_data['other_fields'])){
			$other_fields = sanitize_text_field( serialize( $user_data['other_fields'] ) );
		}

		$updated_rows = $wpdb->query(
			$wpdb->prepare(
				"UPDATE $cart_table
				SET name = %s,
				surname = %s,
				email = %s,
				phone = %s,
				phone_consent = %d,
				location = %s,
				cart_contents = %s,
				cart_total = %0.2f,
				currency = %s,
				time = %s,
				language = %s,
				other_fields = '$other_fields',
				saved_via = $source,
				sms_excluded = 0
				WHERE session_id = %s AND
				(type = %d OR type = %d)",
				sanitize_text_field( $user_data['name'] ),
				sanitize_text_field( $user_data['surname'] ),
				sanitize_email( $user_data['email'] ),
				filter_var( $user_data['phone'], FILTER_SANITIZE_NUMBER_INT),
				sanitize_text_field( $phone_consent ),
				sanitize_text_field( serialize( $user_data['location'] ) ),
				serialize( $cart['product_array'] ),
				sanitize_text_field( $cart['cart_total'] ),
				sanitize_text_field( $cart['cart_currency'] ),
				sanitize_text_field( $cart['current_time'] ),
				sanitize_text_field( $user_data['language'] ),
				$cart['session_id'],
				$admin->get_cart_type('abandoned'),
				$admin->get_cart_type('recovered_pending')
			)
		);

		$this->delete_duplicate_carts( $cart['session_id'], $updated_rows);
		$this->increase_recoverable_cart_count();

		//Saving default coupons if customer has applied any
		if( isset( WC()->cart ) ){
			$applied_coupons = WC()->cart->get_applied_coupons();
			
			if( !empty( $applied_coupons ) ){ //If customer has applied any coupons, save them
				$coupons = new CartBounty_Pro_Coupons();
				$coupons->update_cart_coupon( $code = $applied_coupons, $recovery = 'manual', $step_nr = false, $cart['session_id'] );
			}
		}
	}

	/**
	 * Method shows if the current user is identifiable and the cart can be recovered later
	 *
	 * @since    8.1
	 * @return   Boolean
	 */
	function cart_recoverable(){
		$recoverable = false;
		if( is_user_logged_in() || isset( $_POST["action"]) || $this->cart_identifiable() ){
			$recoverable = true;
		}
		return $recoverable;
	}

	/**
	 * Method returns True in case the current user has got a cart that consists phone or email
	 *
	 * @since    5.0
	 * @return   Boolean
	 */
	function cart_identifiable(){
		global $wpdb;
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$identifiable = false;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$where_sentence = $admin->get_where_sentence('recoverable');
		$cart = $this->read_cart();

		//Checking if we have this abandoned cart in our database already
		$result = $wpdb->get_var($wpdb->prepare(
			"SELECT id
			FROM $cart_table
			WHERE session_id = %s
			$where_sentence",
			$cart['session_id']
		));

		if($result){
			$identifiable = true;
		}

		return $identifiable;
	}

	/**
	 * Method returns available user's data
	 *
	 * @since    5.0
	 * @return   Array
	 */
	function get_user_data(){
		$user_data = array();

		if ( is_user_logged_in() && !isset( $_POST["action"] )){ //If user has signed in and the request is not triggered by checkout fields or Exit Intent
			$current_user = wp_get_current_user(); //Retrieving users data
			//Looking if a user has previously made an order. If not, using default WordPress assigned data
			(isset($current_user->billing_first_name)) ? $name = $current_user->billing_first_name : $name = $current_user->user_firstname; //If/Else shorthand (condition) ? True : False
			(isset($current_user->billing_last_name)) ? $surname = $current_user->billing_last_name : $surname = $current_user->user_lastname;
			(isset($current_user->billing_email)) ? $email = $current_user->billing_email : $email = $current_user->user_email;
			(isset($current_user->billing_phone)) ? $phone = $current_user->billing_phone : $phone = '';
			(isset($current_user->billing_country)) ? $country = $current_user->billing_country : $country = '';
			(isset($current_user->billing_city)) ? $city = $current_user->billing_city : $city = '';
			(isset($current_user->billing_postcode)) ? $postcode = $current_user->billing_postcode : $postcode = '';

			if($country == ''){ //Trying to Geolocate user's country in case it was not found
				$country = WC_Geolocation::geolocate_ip(); //Getting users country from his IP address
				$country = $country['country'];
			}

			$location = array(
				'country' 	=> $country,
				'city' 		=> $city,
				'postcode' 	=> $postcode
			);

			//Try to check if registered user has phone consent data saved and retrieve it
			$customer = new WC_Customer( get_current_user_id() );
			$phone_consent = 0;
			if( $customer->get_meta( 'billing_phone_consent' ) ){
				$phone_consent = 1;
			}

			$user_data = array(
				'name'			=> $name,
				'surname'		=> $surname,
				'email'			=> $email,
				'phone'			=> $phone,
				'phone_consent'	=> $phone_consent,
				'location'		=> $location,
				'other_fields'	=> '',
				'language'		=> get_locale()
			);

		}else{ //Checking if we have values coming from the input fields
			(isset($_POST['cartbounty_pro_name'])) ? $name = $_POST['cartbounty_pro_name'] : $name = ''; //If/Else shorthand (condition) ? True : False
			(isset($_POST['cartbounty_pro_surname'])) ? $surname = $_POST['cartbounty_pro_surname'] : $surname = '';
			(isset($_POST['cartbounty_pro_email'])) ? $email = $_POST['cartbounty_pro_email'] : $email = '';
			(isset($_POST['cartbounty_pro_phone'])) ? $phone = $_POST['cartbounty_pro_phone'] : $phone = '';
			(isset($_POST['cartbounty_pro_phone_consent'])) ? $phone_consent = $_POST['cartbounty_pro_phone_consent'] : $phone_consent = $this->get_saved_cart_data( 'phone_consent' ); //If data not provided, check if we have this value already saved in te database (required when saving data with Exit Intent)
			(isset($_POST['cartbounty_pro_country'])) ? $country = $_POST['cartbounty_pro_country'] : $country = '';
			(isset($_POST['cartbounty_pro_city']) && $_POST['cartbounty_pro_city'] != '') ? $city = $_POST['cartbounty_pro_city'] : $city = '';
			(isset($_POST['cartbounty_pro_billing_company'])) ? $company = $_POST['cartbounty_pro_billing_company'] : $company = '';
			(isset($_POST['cartbounty_pro_billing_address_1'])) ? $address_1 = $_POST['cartbounty_pro_billing_address_1'] : $address_1 = '';
			(isset($_POST['cartbounty_pro_billing_address_2'])) ? $address_2 = $_POST['cartbounty_pro_billing_address_2'] : $address_2 = '';
			(isset($_POST['cartbounty_pro_billing_state'])) ? $state = $_POST['cartbounty_pro_billing_state'] : $state = '';
			(isset($_POST['cartbounty_pro_billing_postcode'])) ? $postcode = $_POST['cartbounty_pro_billing_postcode'] : $postcode = '';
			(isset($_POST['cartbounty_pro_shipping_first_name'])) ? $shipping_name = $_POST['cartbounty_pro_shipping_first_name'] : $shipping_name = '';
			(isset($_POST['cartbounty_pro_shipping_last_name'])) ? $shipping_surname = $_POST['cartbounty_pro_shipping_last_name'] : $shipping_surname = '';
			(isset($_POST['cartbounty_pro_shipping_company'])) ? $shipping_company = $_POST['cartbounty_pro_shipping_company'] : $shipping_company = '';
			(isset($_POST['cartbounty_pro_shipping_country'])) ? $shipping_country = $_POST['cartbounty_pro_shipping_country'] : $shipping_country = '';
			(isset($_POST['cartbounty_pro_shipping_address_1'])) ? $shipping_address_1 = $_POST['cartbounty_pro_shipping_address_1'] : $shipping_address_1 = '';
			(isset($_POST['cartbounty_pro_shipping_address_2'])) ? $shipping_address_2 = $_POST['cartbounty_pro_shipping_address_2'] : $shipping_address_2 = '';
			(isset($_POST['cartbounty_pro_shipping_city'])) ? $shipping_city = $_POST['cartbounty_pro_shipping_city'] : $shipping_city = '';
			(isset($_POST['cartbounty_pro_shipping_state'])) ? $shipping_state = $_POST['cartbounty_pro_shipping_state'] : $shipping_state = '';
			(isset($_POST['cartbounty_pro_shipping_postcode'])) ? $shipping_postcode = $_POST['cartbounty_pro_shipping_postcode'] : $shipping_postcode = '';
			(isset($_POST['cartbounty_pro_order_comments'])) ? $comments = $_POST['cartbounty_pro_order_comments'] : $comments = '';
			(isset($_POST['cartbounty_pro_create_account'])) ? $create_account = $_POST['cartbounty_pro_create_account'] : $create_account = '';
			(isset($_POST['cartbounty_pro_ship_elsewhere'])) ? $ship_elsewhere = $_POST['cartbounty_pro_ship_elsewhere'] : $ship_elsewhere = '';
			(isset($_POST['cartbounty_pro_language'])) ? $language = $_POST['cartbounty_pro_language'] : $language = '';
			
			$other_fields = array(
				'cartbounty_pro_billing_company' 		=> $company,
				'cartbounty_pro_billing_address_1' 		=> $address_1,
				'cartbounty_pro_billing_address_2' 		=> $address_2,
				'cartbounty_pro_billing_state' 			=> $state,
				'cartbounty_pro_shipping_first_name' 	=> $shipping_name,
				'cartbounty_pro_shipping_last_name' 	=> $shipping_surname,
				'cartbounty_pro_shipping_company' 		=> $shipping_company,
				'cartbounty_pro_shipping_country' 		=> $shipping_country,
				'cartbounty_pro_shipping_address_1' 	=> $shipping_address_1,
				'cartbounty_pro_shipping_address_2' 	=> $shipping_address_2,
				'cartbounty_pro_shipping_city' 			=> $shipping_city,
				'cartbounty_pro_shipping_state' 		=> $shipping_state,
				'cartbounty_pro_shipping_postcode' 		=> $shipping_postcode,
				'cartbounty_pro_order_comments' 		=> $comments,
				'cartbounty_pro_create_account' 		=> $create_account,
				'cartbounty_pro_ship_elsewhere' 		=> $ship_elsewhere
			);

			if($country == ''){ //Trying to Geolocate user's country in case it was not found
				$country = WC_Geolocation::geolocate_ip(); //Getting users country from his IP address
				$country = $country['country'];
			}
			
			$location = array(
				'country' 	=> $country,
				'city' 		=> $city,
				'postcode' 	=> $postcode
			);

			$user_data = array(
				'name'			=> $name,
				'surname'		=> $surname,
				'email'			=> $email,
				'phone'			=> $phone,
				'phone_consent'	=> $phone_consent,
				'location'		=> $location,
				'other_fields'	=> $other_fields,
				'language'		=> $language
			);
		}

		return $user_data;
	}

	/**
	 * Method checks if recaptcha has been enabled and determines if the cart should be saved or not
	 * Returns true if all is ok and false in case there was an issue
	 *
	 * @since    8.1
	 * @return   boolean
	 */
	function check_recaptcha(){
		$result = true;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$recaptcha = $admin->get_recaptcha();

		//Checking if we have reCAPTCHA token coming from the form
		if(isset($_POST['cartbounty_pro_recaptcha_token'])){
			//Send request to the reCAPTCHA validation service
			$recaptcha_params = array(
				'secret' => $recaptcha['secret_key'],
				'response' => sanitize_text_field($_POST["cartbounty_pro_recaptcha_token"])
			);
			$query = esc_url_raw(add_query_arg($recaptcha_params, RECAPTCHA_SITE_API));
			$response = wp_remote_get($query, array('timeout' => 10, 'sslverify' => false));
			$response_body = json_decode(wp_remote_retrieve_body($response));

			//If this request was not a valid reCAPTCHA token for this site - not saving abandoned cart and cleaning any previously saved cart for this user
			if(!$response_body->success){
				$admin->log('info', esc_html__('Not saving abandoned cart: Request did not include a valid reCAPTCHA token for this site.', 'woo-save-abandoned-carts'));
				$admin->clear_cart_data();
				$result = false;
			}

			//Setting the default minimum required reCAPTCHA score to be 0.2
			$recaptcha_minimum_allowed_score = esc_html(apply_filters( 'cartbounty_pro_recaptcha_minimum_allowed_score', 0.2));

			if(empty($response_body->action)){ //In case no data received from response about action, we make sure to set it so we would not get an error notice
				$response_body->action = '';
			}

			//If this request does not return our action defined in token or if the score is lower than what is set - not saving abandoned cart
			if(($response_body->action != 'cartbounty_pro_abandoned_cart_checkout' && $response_body->action != 'cartbounty_pro_abandoned_cart_tool') || $response_body->score < $recaptcha_minimum_allowed_score ){
				if(empty($response_body->score)){
					$response_body->score = 'null';
				}
				$admin->clear_cart_data();
				$admin->log('notice', sprintf(esc_html__("Not saving abandoned cart: Minimum allowed score is %s, but visitor's score was %s.", 'woo-save-abandoned-carts'), esc_html( $recaptcha_minimum_allowed_score ), esc_html( $response_body->score ) ) );
				$result = false;
			}else{
				$admin->log('info', sprintf(esc_html__("Saving abandoned cart: Successful validation by reCAPTCHA. Visitor's score was %s.", 'woo-save-abandoned-carts'), esc_html( $response_body->score ) ) );
			}

		}else{ //In case no token received, meaning that function is not triggered by any of the forms CartBounty is listening to
			if($this->cart_recoverable()){
				if($recaptcha['enabled'] && !is_user_logged_in()){
					//In case if the token was not received, output an error message
					$admin->log('info', esc_html__('Saving cart, but reCAPTCHA token was not received.', 'woo-save-abandoned-carts'));
					$result = true;
				}
			}
		}

		return $result;
	}

	/**
	 * Method sets CartBounty session id value
	 *
	 * @since    5.0
	 */
	function set_cartbounty_session($session_id){
		if(!WC()->session->get('cartbounty_pro_session_id')){ //In case browser session is not set, we make sure it gets set
			WC()->session->set('cartbounty_pro_session_id', $session_id); //Storing session_id in WooCommerce session
		}
	}

	/**
	 * Method checks if current user session ID also exists in the database and the cart has not been paid for (type = abandoned or recovered_pending)
	 *
	 * @since    4.0
	 * @return   boolean
	 * @param    $session_id    Session ID
	 */
	function cart_saved( $session_id ){
		$saved = false;
		if( $session_id !== NULL ){
			global $wpdb;
			$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
			$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;

			//Checking if we have this abandoned cart in our database already
			$result = $wpdb->get_var(
				$wpdb->prepare(
					"SELECT session_id
					FROM $cart_table
					WHERE session_id = %s AND
					(type = %d OR type = %d)",
					$session_id,
					$admin->get_cart_type('abandoned'),
					$admin->get_cart_type('recovered_pending')
				)
			);

			if($result){
				$saved = true;
			}
		}

		return $saved;
	}

	/**
	 * Method updates abandoned cart session from unknown session customer_id to known one in case if the user logs in
	 *
	 * @since    6.5
	 */
	function update_logged_customer_id(){
		if(is_user_logged_in()){ //If a user is logged in
			if(!WC()->session){ //If session does not exist, exit function
				return;
			}
			$customer_id = WC()->session->get_customer_id();

			if( WC()->session->get('cartbounty_pro_session_id') !== NULL && WC()->session->get('cartbounty_pro_session_id') !== $customer_id){ //If session is set and it is different from the one that currently is assigned to the customer
				global $wpdb;
				$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;

				//Updating session ID to match the one of a logged in user
				$wpdb->update(
					$cart_table,
					array('session_id' => $customer_id),
					array('session_id' => WC()->session->get('cartbounty_pro_session_id')),
					array('%s'),
					array('%s')
				);

				WC()->session->set('cartbounty_pro_session_id', $customer_id);
				$cart = $this->read_cart();
				$user_data = $this->get_user_data();
				$this->update_cart_and_user_data($cart, $user_data); //Updating logged in user cart data so we do not have anything that was entered in the checkout form prior the user signed in

			}else{
				return;
			}

		}else{
			return;
		}
	}

	/**
	 * Method deletes duplicate abandoned carts from the database
	 *
	 * @since    6.5
	 * @param    $session_id			Session ID
	 * @param    $duplicate_count		Number of duplicate carts
	 */
	private function delete_duplicate_carts( $session_id, $duplicate_count ){
		global $wpdb;
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;

		if($duplicate_count){ //If we have updated at least one row
			if($duplicate_count > 1){ //Checking if we have updated more than a single row to know if there were duplicates
				$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
				$where_sentence = $admin->get_where_sentence('ghost');
				//First delete all duplicate ghost carts
				$deleted_duplicate_ghost_carts = $wpdb->query(
					$wpdb->prepare(
						"DELETE FROM $cart_table
						WHERE session_id = %s
						$where_sentence",
						$session_id
					)
				);

				$limit = $duplicate_count - $deleted_duplicate_ghost_carts - 1;
				if($limit < 1){
					$limit = 0;
				}

				$deleted_duplicate = $wpdb->query( //Leaving one cart remaining that can be identified
					$wpdb->prepare(
						"DELETE FROM $cart_table
						WHERE session_id = %s AND
						type != %d
						ORDER BY id DESC
						LIMIT %d",
						$session_id,
						$admin->get_cart_type('recovered'),
						$limit
					)
				);
			}
		}
	}

	/**
	 * Method builds and returns an array of cart products and their quantities, car total value, currency, time, session id, mail status
	 *
	 * @since    4.0
	 * @return   Array
	 */
	function read_cart(){

		if( !WC()->cart ){ //Exit if Woocommerce cart has not been initialized
			return;
		}

		//Retrieving cart total value and currency
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$cart_total = WC()->cart->total;
		$cart_currency = get_woocommerce_currency();
		$current_time = current_time( 'mysql', false ); //Retrieving current time
		$session_id = WC()->session->get( 'cartbounty_pro_session_id' ); //Check if the session is already set

		if( empty( $session_id ) ){ //If session value does not exist - set one now
			$session_id = WC()->session->get_customer_id(); //Retrieving customer ID from WooCommerce sessions variable
		}

		if( WC()->session->get( 'cartbounty_pro_from_link' ) && WC()->session->get( 'cartbounty_pro_session_id' ) ){ //In case users returns from abandoned cart email link
			$session_id = WC()->session->get( 'cartbounty_pro_session_id' );
		}

		//Retrieving cart
		$products = WC()->cart->get_cart_contents();
		$product_array = array();
				
		foreach( $products as $key => $product ){
			$item = wc_get_product( $product['data']->get_id() );
			$product_title = $item->get_title();
			$product_quantity = $product['quantity'];
			$product_variation_price = '';
			$product_tax = '';

			if( isset( $product['line_total'] ) ){
				$product_variation_price = $product['line_total'];
			}

			if( isset( $product['line_tax'] ) ){ //If we have taxes, add them to the price
				$product_tax = $product['line_tax'];
			}
			
			// Handling product variations
			if( $product['variation_id'] ){ //If user has chosen a variation
				$single_variation = new WC_Product_Variation( $product['variation_id'] );
		
				//Handling variable product title output with attributes
				$product_attributes = $this->attribute_slug_to_title( $single_variation->get_variation_attributes() );
				$product_variation_id = $product['variation_id'];

			}else{
				$product_attributes = false;
				$product_variation_id = '';
			}

			$product_data = array(
				'product_title' => $product_title . $product_attributes,
				'quantity' => $product_quantity,
				'product_id' => $product['product_id'],
				'product_variation_id' => $product_variation_id,
				'product_variation_price' => $product_variation_price,
				'product_tax' => $product_tax
			);

			//Handle WooCommerce Product Bundles plugin data
			if ( class_exists( 'WC_Bundles' ) ) { //If WooCommerce Product Bundles plugin active
				if( !wc_pb_is_bundled_cart_item( $product ) ){ //If cart item is not part of a bundle (looking to find only bundle parent item)
					if( isset( $product['stamp'] ) ){ //If bundle parent has data about bundled items
						$bundled_item_data = $product['stamp'];
						$product_data['bundle_data'] = $bundled_item_data;
						$bundle_price = 0;
						$bundle_tax = 0;
						$bundled_cart_items = wc_pb_get_bundled_cart_items( $product );

						//Looping through bundle sub-items to get price of the main bundle item
						foreach ( $bundled_cart_items as $key => $bundled_cart_item ) {

							if( isset( $bundled_cart_item['line_total'] ) ){
								$bundled_cart_item_price = $bundled_cart_item['line_total'];
							}

							if( isset( $bundled_cart_item['line_tax'] ) ){ //If we have taxes, add them to the price
								$bundled_cart_item_tax = $bundled_cart_item['line_tax'];
							}

							$bundle_price = $bundle_price + $bundled_cart_item_price;
							$bundle_tax = $bundle_tax + $bundled_cart_item_tax;
						}

						$product_data['product_variation_price'] = $product_variation_price + $bundle_price;
						$product_data['product_tax'] = $product_tax + $bundle_tax;
					}

				}else{ //Skip adding individual bundle sub-items to the cart
					continue;
				}
			}

			$product_data = $admin->add_custom_product_fields( $product, $product_data ); //Handle custom product fields plugins
			$product_array[] = $product_data;
		}

		return $results_array = array(
			'cart_total' 	=> $cart_total,
			'cart_currency' => $cart_currency,
			'current_time' 	=> $current_time,
			'session_id' 	=> $session_id,
			'product_array' => $product_array
		);
	}

	/**
	 * Method returns product attributes
	 *
	 * @since    1.4.1
	 * @return   String
	 * @param    $product_variations    Product variations - array
	 */
	public function attribute_slug_to_title( $product_variations ) {
		$attribute_array = array();
		
		if($product_variations){
			foreach($product_variations as $product_variation_key => $product_variation_name){
				$value = '';
				if ( taxonomy_exists( esc_attr( str_replace( 'attribute_', '', $product_variation_key )))){
					$term = get_term_by( 'slug', $product_variation_name, esc_attr( str_replace( 'attribute_', '', $product_variation_key )));
					if (!is_wp_error($term) && !empty($term->name)){
						$value = $term->name;
						if(!empty($value)){
							$attribute_array[] = $value;
						}
					}
				}else{
					$value = apply_filters( 'woocommerce_variation_option_name', $product_variation_name );
					if(!empty($value)){
						$attribute_array[] = $value;
					}
				}
			}
			
			//Generating attribute output			
			$total_variations = count($attribute_array);
			$increment = 0;
			$product_attribute = '';
			foreach($attribute_array as $attribute){
				if($increment === 0 && $increment != $total_variations - 1){ //If this is first variation and we have multiple variations
					$colon = ': ';
					$comma = ', ';
				}
				elseif($increment === 0 && $increment === $total_variations - 1){ //If we have only one variation
					$colon = ': ';
					$comma = false;
				}
				elseif($increment === $total_variations - 1) { //If this is the last variation
					$comma = '';
					$colon = false;
				}else{
					$comma = ', ';
					$colon = false;
				}
				$product_attribute .= $colon . $attribute . $comma;
				$increment++;
			}
			return $product_attribute;
		}
		else{
			return;
		}
	}
	
	/**
	 * Method restores previous Checkout form data for users who are not registered
	 *
	 * @since    3.0
	 * @return   Input field values
	 * @param    $fields    Checkout fields - array
	 */
	public function restore_input_data( $fields = array() ) {

		if(is_account_page()){ //In case this is user's account page, do not restore data. Added since some plugins change the My account form and may trigger this function unnecessarily
			return $fields;
		}
		
		global $wpdb;
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$cart = $this->read_cart();

		$this->update_logged_customer_id(); //If current user had an abandoned cart before - restore session ID (in case of user switching)

		//Retrieve a single row with current customer ID
		$row = $wpdb->get_row($wpdb->prepare(
			"SELECT *
			FROM $cart_table
			WHERE session_id = %s",
			$cart['session_id'])
		);

		if($row){ //If we have a user with such session ID in the database

			$other_fields = @unserialize($row->other_fields);

			if(is_serialized($row->location)){ //Since version 6.8
	            $location_data = unserialize($row->location);
	            $country = $location_data['country'];
	            $city = $location_data['city'];
	            $postcode = $location_data['postcode'];

	        }else{ //Prior version 6.8. Will be removed in future releases
	        	$parts = explode(',', $row->location); //Splits the Location field into parts where there are commas
	            if (count($parts) > 1) {
	                $country = $parts[0];
	                $city = trim($parts[1]); //Trim removes white space before and after the string
	            }
	            else{
	                $country = $parts[0];
	                $city = '';
	            }

	            $postcode = '';
                if(isset($other_fields['cartbounty_pro_billing_postcode'])){
                    $postcode = $other_fields['cartbounty_pro_billing_postcode'];
                }
	        }

			(empty( $_POST['billing_first_name'])) ? $_POST['billing_first_name'] = sprintf('%s', esc_html($row->name)) : '';
			(empty( $_POST['billing_last_name'])) ? $_POST['billing_last_name'] = sprintf('%s', esc_html($row->surname)) : '';
			(empty( $_POST['billing_country'])) ? $_POST['billing_country'] = sprintf('%s', esc_html($country)) : '';
			(empty( $_POST['billing_city'])) ? $_POST['billing_city'] = sprintf('%s', esc_html($city)) : '';
			(empty( $_POST['billing_phone'])) ? $_POST['billing_phone'] = sprintf('%s', esc_html($row->phone)) : '';
			(empty( $_POST['billing_phone_consent'])) ? $_POST['billing_phone_consent'] = sprintf('%s', esc_html($row->phone_consent)) : '';
			(empty( $_POST['billing_email'])) ? $_POST['billing_email'] = sprintf('%s', esc_html($row->email)) : '';
			(empty( $_POST['billing_postcode'])) ? $_POST['billing_postcode'] = sprintf('%s', esc_html($postcode)) : '';

			if($other_fields){
				(empty( $_POST['billing_company'])) ? $_POST['billing_company'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_billing_company'])) : '';
				(empty( $_POST['billing_address_1'])) ? $_POST['billing_address_1'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_billing_address_1'])) : '';
				(empty( $_POST['billing_address_2'])) ? $_POST['billing_address_2'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_billing_address_2'])) : '';
				(empty( $_POST['billing_state'])) ? $_POST['billing_state'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_billing_state'])) : '';
				(empty( $_POST['shipping_first_name'])) ? $_POST['shipping_first_name'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_first_name'])) : '';
				(empty( $_POST['shipping_last_name'])) ? $_POST['shipping_last_name'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_last_name'])) : '';
				(empty( $_POST['shipping_company'])) ? $_POST['shipping_company'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_company'])) : '';
				(empty( $_POST['shipping_country'])) ? $_POST['shipping_country'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_country'])) : '';
				(empty( $_POST['shipping_address_1'])) ? $_POST['shipping_address_1'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_address_1'])) : '';
				(empty( $_POST['shipping_address_2'])) ? $_POST['shipping_address_2'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_address_2'])) : '';
				(empty( $_POST['shipping_city'])) ? $_POST['shipping_city'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_city'])) : '';
				(empty( $_POST['shipping_state'])) ? $_POST['shipping_state'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_state'])) : '';
				(empty( $_POST['shipping_postcode'])) ? $_POST['shipping_postcode'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_shipping_postcode'])) : '';
				(empty( $_POST['order_comments'])) ? $_POST['order_comments'] = sprintf('%s', esc_html($other_fields['cartbounty_pro_order_comments'])) : '';
			}
			
			//Checking if Create account should be checked or not
			if(isset($other_fields['cartbounty_pro_create_account'])){
				if($other_fields['cartbounty_pro_create_account']){
					add_filter( 'woocommerce_create_account_default_checked', '__return_true' );
				}
			}

			//Checking if Ship to a different location must be checked or not
			if(isset($other_fields['cartbounty_pro_ship_elsewhere'])){
				if($other_fields['cartbounty_pro_ship_elsewhere']){
					add_filter( 'woocommerce_ship_to_different_address_checked', '__return_true' );
				}
			}
		}
		
		return $fields;
	}

	/**
	 * Method saves and updates total count of captured recoverable abandoned carts
	 *
	 * @since    8.1
	 */
	function increase_recoverable_cart_count(){
		if(!WC()->session){ //If session does not exist, exit function
			return;
		}
		if(WC()->session->get('cartbounty_pro_recoverable_count_increased') || WC()->session->get('cartbounty_pro_from_link')){ //Exit fnction in case we already have run this once
			return;
		}
		update_option('cartbounty_pro_recoverable_cart_count', get_option('cartbounty_pro_recoverable_cart_count') + 1); //Increasing the count by one abandoned cart
		WC()->session->set('cartbounty_pro_recoverable_count_increased', 1);

		if(WC()->session->get('cartbounty_pro_ghost_count_increased')){ //In case we previously increased ghost cart count, we must now reduce it as it has been turned to recoverable
			$this->decrease_ghost_cart_count( 1 );
		}
	}

	/**
	 * Method saves and updates total count of captured ghost carts
	 *
	 * @since    8.1
	 */
	function increase_ghost_cart_count(){
		if(!WC()->session){ //If session does not exist, exit function
			return;
		}
		if(WC()->session->get('cartbounty_pro_ghost_count_increased')){ //Exit fnction in case we already have run this once
			return;
		}
		update_option('cartbounty_pro_ghost_cart_count', get_option('cartbounty_pro_ghost_cart_count') + 1);
		WC()->session->set('cartbounty_pro_ghost_count_increased', 1);
	}

	/**
	 * Method saves and updates total count of recovered carts
	 *
	 * @since    9.2.3
	 * @param    boolean    $order_status_changed            If function is triggered from changing order status or not
	 */
	function increase_recovered_cart_count( $order_status_changed ){
		if( ! $order_status_changed){ //If order status has not changed - check if session Exists
			if(!WC()->session){ //If session does not exist, exit function
				return;
			}

			if(WC()->session->get('cartbounty_pro_recovered_count_increased')){ //Exit function in case we already have run this once
				return;
			}

			WC()->session->set('cartbounty_pro_recovered_count_increased', 1);
		}
		update_option('cartbounty_pro_recovered_cart_count', get_option('cartbounty_pro_recovered_cart_count') + 1);
	}

	/**
	 * Method decreases the total count of captured abandoned carts
	 *
	 * @since    8.1
	 * @param    $count    Cart number - integer 
	 */
	function decrease_recoverable_cart_count( $count ){
		update_option('cartbounty_pro_recoverable_cart_count', get_option('cartbounty_pro_recoverable_cart_count') - $count); //Decreasing the count by one abandoned cart
		delete_transient( 'cartbounty_pro_recoverable_cart_count' );
	}

	/**
	 * Method decreases the total count of captured ghost carts
	 *
	 * @since    8.1
	 * @param    $count    Cart number - integer 
	 */
	function decrease_ghost_cart_count( $count ){
		update_option('cartbounty_pro_ghost_cart_count', get_option('cartbounty_pro_ghost_cart_count') - $count);
	}

	/**
	 * Outputing email form if a user who is not logged in wants to leave with a full shopping cart
	 *
	 * @since    4.0
	 */
	function display_exit_intent_form(){
		if (!class_exists('WooCommerce')) { //If WooCommerce does not exist
			return;
		}

		if(!$this->tool_enabled( 'exit_intent' ) || !WC()->cart){ //If Exit Intent disabled or WooCommerce cart does not exist
			return;
		}

		$current_user_is_admin = current_user_can( 'manage_options' );
		$output = $this->build_exit_intent_output( $current_user_is_admin ); //Creating the Exit Intent output
		echo $output;
		echo "<script>localStorage.setItem( 'cartbounty_pro_product_count', " . $this->get_cart_product_count() . " )</script>";
	}

	/**
	 * Adding Early capture request to the page
	 *
	 * @since    9.1
	 */
	function display_early_capture_form(){
		if (!class_exists('WooCommerce')) { //If WooCommerce does not exist
			return;
		}

		if(!$this->tool_enabled( 'early_capture' )){ //If Early capture disabled
			return;
		}

		$output = $this->build_early_capture_output(); //Creating the Early capture output
		if (isset( $_POST["cartbounty_pro_show_early_capture"])) { //When function triggered using Ajax Add to Cart
			return wp_send_json_success($output); //Sending Output to Javascript function
		}		
	}

	/**
	 * Checking if cart is empty and sending result to Ajax function
	 *
	 * @since    4.0
	 * @return   boolean
	 */
	function check_empty_cart(){
		if(!WC()->cart) return;

		if( WC()->cart->get_cart_contents_count() == 0 ){ //If the cart is empty
			return wp_send_json_success('true'); //Sending successful output to Javascript function

		}else{
			return wp_send_json_success('false');
		}
	}

	/**
	 * Building the Exit Intent output
	 *
	 * @since    4.0
	 * @return   string
	 * @param    $current_user_is_admin    If the current user has Admin rights or not - boolean
	 */
	function build_exit_intent_output( $current_user_is_admin ){
		global $wpdb;
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$cart = $this->read_cart();
		$exit_intent_heading = $admin->get_exit_intent_heading();
		$exit_intent_content = $admin->get_exit_intent_content();
		$heading = apply_filters( 'wpml_translate_single_string', $exit_intent_heading['value'], 'woo-save-abandoned-carts', $exit_intent_heading['name'] );
		$content = apply_filters( 'wpml_translate_single_string', $exit_intent_content['value'], 'woo-save-abandoned-carts', $exit_intent_content['name'] );
		$main_color = get_option('cartbounty_pro_exit_intent_main_color');
		$inverse_color = get_option('cartbounty_pro_exit_intent_inverse_color');
		$where_sentence = $admin->get_where_sentence('recoverable');
		$phone_tools_consent = $admin->get_phone_tools_consent();
		$tools_consent = apply_filters( 'wpml_translate_single_string', $phone_tools_consent['value'], 'woo-save-abandoned-carts', $phone_tools_consent['name'] );

		if(!$main_color){
			$main_color = '#e3e3e3';
		}
		if(!$inverse_color){
			$inverse_color = $this->invert_color($main_color);
		}

		//Retrieve a single row with current customer ID
		$row = $wpdb->get_row($wpdb->prepare(
			"SELECT *
			FROM  $cart_table
			WHERE session_id = %s
			$where_sentence",
			$cart['session_id'])
		);

		if($row && !$current_user_is_admin){ //Exit if cart already saved and the current user is not admin
			return;
		}

		//Prepare Exit Intent image
		$image_id = get_option('cartbounty_pro_exit_intent_image');
		$image_url = $this->get_plugin_url() . '/public/assets/abandoned-shopping-cart.gif';
		if($image_id){
			$image = wp_get_attachment_image_src( $image_id, 'full' );
			if(is_array($image)){
				$image_url = $image[0];
			}
		}

		$type = get_option('cartbounty_pro_exit_intent_field_type');

		$args = apply_filters(
			'cartbounty_pro_exit_intent_args',
			array(
				'image_url' => $image_url,
				'heading' => $heading,
				'content' => $content,
				'type' => $type,
				'main_color' => $main_color,
				'inverse_color' => $inverse_color,
				'phone_consent_on' => $admin->phone_consent_enabled(),
				'phone_tools_consent' => $tools_consent
			)
		);

		return $this->get_template( 'cartbounty-pro-exit-intent.php', $args );
		
	}

	/**
	 * Building Early capture output
	 *
	 * @since    9.1
	 * @return   string
	 */
	function build_early_capture_output(){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		$early_capture_heading = $admin->get_early_capture_heading();
		$heading = apply_filters( 'wpml_translate_single_string', $early_capture_heading['value'], 'woo-save-abandoned-carts', $early_capture_heading['name'] );
		$main_color = get_option('cartbounty_pro_early_capture_main_color');
		$inverse_color = get_option('cartbounty_pro_early_capture_inverse_color');
		$style = get_option('cartbounty_pro_early_capture_type');
		$phone_tools_consent = $admin->get_phone_tools_consent();
		$tools_consent = apply_filters( 'wpml_translate_single_string', $phone_tools_consent['value'], 'woo-save-abandoned-carts', $phone_tools_consent['name'] );

		if(!$main_color && $style != 2){
			$main_color = '#ffffff';
		}elseif(!$main_color){
			$main_color = '#e3e3e3';
		}
		
		if(!$main_color){
			$main_color = '#e3e3e3';
		}
		if(!$inverse_color){
			$inverse_color = $this->invert_color($main_color);
		}

		$type = get_option('cartbounty_pro_early_capture_field_type');
		$style = $this->early_capture_style();

		$args = apply_filters(
			'cartbounty_pro_early_capture_args',
			array(
				'heading' => $heading,
				'type' => $type,
				'style' => $style,
				'main_color' => $main_color,
				'inverse_color' => $inverse_color,
				'phone_consent_on' => $admin->phone_consent_enabled(),
				'phone_tools_consent' => $tools_consent
			)
		);

		//In case the function is called via Ajax Add to Cart button
		//We must add wp_die() or otherwise the function does not return anything
		if (isset( $_POST["cartbounty_pro_show_early_capture"])){ 
			$output = $this->get_template( 'cartbounty-pro-early-capture.php', $args );
			die();
		}else{
			return $this->get_template( 'cartbounty-pro-early-capture.php', $args );
		}
	}

	/**
	 * Checking if a tool is enabled or not
	 *
	 * @since    9.1
	 * @return   boolean
	 * @param    string    $tool 					Tool that is being checked
	 * @param    boolean   $ignore_logged_users 	Weather logged in users should be ignored
	 */
	function tool_enabled( $tool, $ignore_logged_users = false ){
		$admin = new CartBounty_Pro_Admin(CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER);
		if(!$admin->check_license()){//Exit if license key not valid
			return false;
		}else{
			switch ( $tool ) {
				case 'exit_intent':
					$tool_enabled = get_option('cartbounty_pro_exit_intent_status');
					$test_mode_on = get_option('cartbounty_pro_exit_intent_test_mode');
					break;

				case 'early_capture':
					$tool_enabled = get_option('cartbounty_pro_early_capture_status');
					$test_mode_on = get_option('cartbounty_pro_early_capture_test_mode');
					break;

				case 'tab_notification':
					$tool_enabled = get_option('cartbounty_pro_tab_notification_status');
					$test_mode_on = get_option('cartbounty_pro_tab_notification_test_mode');
					break;
			}

			$current_user_is_admin = current_user_can( 'manage_options' );

			if($test_mode_on && $current_user_is_admin){
				//Outputing tool for Testing purposes to Administrators
				return true;
			}elseif( ( $tool_enabled && !$test_mode_on && !is_user_logged_in() ) || ( $tool_enabled && !$test_mode_on && $ignore_logged_users ) ){
				//Outputing tool for all users who are not logged in
				return true;
			}else{
				//Tool is not enabled
				return false;
			}
		}
	}

	/**
	 * Getting inverse color from the given one
	 *
	 * @since    4.0
	 * @return   string
	 * @param    $color    Color code - string
	 */
	function invert_color( $color ){
	    $color = str_replace('#', '', $color);
	    if (strlen($color) != 6){ return '000000'; }
	    $rgb = '';
	    for ($x=0;$x<3;$x++){
	        $c = 255 - hexdec(substr($color,(2*$x),2));
	        $c = ($c < 0) ? 0 : dechex($c);
	        $rgb .= (strlen($c) < 2) ? '0'.$c : $c;
	    }
	    return '#'.$rgb;
	}

	/**
	 * Method returns a value that informs about which Exit Intent style is used
	 *
	 * @since    4.0
	 */
	function exit_intent_style(){
		$exit_intent_type_value = 'cartbounty-pro-ei-center'; //Setting default class
		$exit_intent_type = get_option('cartbounty_pro_exit_intent_type');

		if($exit_intent_type == 2){
			$exit_intent_type_value = 'cartbounty-pro-ei-left';
		}elseif($exit_intent_type == 3){
			$exit_intent_type_value = 'cartbounty-pro-ei-fullscreen';
		}

		return $exit_intent_type_value;
	}

	/**
	 * Method returns a value that informs about which Early capture style is used
	 *
	 * @since    9.1
	 */
	function early_capture_style(){
		$early_capture_type_value = 'cartbounty-pro-ec-center-near-button'; //Setting default class
		$early_capture_type = get_option('cartbounty_pro_early_capture_type');

		if($early_capture_type == 2){
			$early_capture_type_value = 'cartbounty-pro-ec-center';
		}

		return $early_capture_type_value;
	}

	/**
	 * Get the plugin url.
	 *
	 * @since    4.0
	 * @return   string
	 */
	public function get_plugin_url() {
		return plugins_url( '/', dirname(__FILE__) );
	}

	/**
	 * Locating template file.
	 * Method returns the path to the template
	 *
	 * Search Order:
	 * 1. /themes/theme/templates/emails/$template_name
	 * 2. /themes/theme/templates/$template_name
	 * 3. /themes/theme/$template_name
	 * 4. /plugins/woo-save-abandoned-carts-pro/templates/$template_name
	 *
	 * @since    4.0
	 * @return   string
	 * @param 	 string    $template_name - template to load.
	 * @param 	 string    $string $template_path - path to templates.
	 * @param    string    $default_path - default path to template files.
	 */
	function get_template_path( $template_name, $template_path = '', $default_path = '' ){
		$search_array = array();

		// Set variable to search folder of theme.
		if ( ! $template_path ) :
			$template_path = 'templates/';
		endif;

		//Add paths to look for template files
		$search_array[] = $template_path . 'emails/' . $template_name; 
		$search_array[] = $template_path . $template_name;
		$search_array[] = $template_name;

		// Search template file in theme folder.
		$template = locate_template( $search_array );

		// Set default plugin templates path.
		if ( ! $default_path ) :
			$default_path = plugin_dir_path( __FILE__ ) . '../templates/'; // Path to the template folder
		endif;

		// Get plugins template file.
		if ( ! $template ) :
			$template = $default_path . $template_name;
		endif;
		return apply_filters( 'get_template_path', $template, $template_name, $template_path, $default_path );
	}

	/**
	 * Get the template.
	 *
	 * @since    4.0
	 *
	 * @param    string    $template_name - template to load.
	 * @param    array     $args - args passed for the template file.
	 * @param    string    $string $template_path - path to templates.
	 * @param    string    $default_path - default path to template files.
	 */
	function get_template( $template_name, $args = array(), $tempate_path = '', $default_path = '' ) {
		if ( is_array( $args ) && isset( $args ) ){
			extract( $args );
		}
		$template_file = $this->get_template_path($template_name, $tempate_path, $default_path);
		if ( ! file_exists( $template_file ) ){ //Handling error output in case template file does not exist
			_doing_it_wrong( __FUNCTION__, sprintf( '<code>%s</code> does not exist.', esc_html( $template_file ) ), '9.1' );
			return;
		}
		include $template_file;
	}
	
	/**
	 * Check weather we must exclude ghost cart from being saved. By default all countries are excluded if ghost cart exclusion is enabled.
	 *
	 * @since    9.2
	 * @return 	 Boolean
	 */
	function exclude_ghost_cart(){
		$exclude = false;
		if(get_option('cartbounty_pro_exclude_ghost_carts')){ //If ghost cart exclusion enabled
			$exclude = true;
			if(get_option('cartbounty_pro_allowed_countries')){ //In case at least one country is selected in the exclusions list
				if($this->is_country_allowed()){ //In case the country is in the exception list of countries - we should allow to save this ghost cart
					$exclude = false;
				}
			}
		}
		return $exclude;
	}

	/**
	 * Check weather current user is from an allowed country or not
	 *
	 * @since    9.2
	 * @return 	 Boolean
	 */
	function is_country_allowed(){
		$allowed = false;
		$allowed_countries = get_option('cartbounty_pro_allowed_countries'); //Retrieving array of allowed countries
		$user_data = $this->get_user_data();
		if(isset($user_data['location'])){
			$location = $user_data['location'];
			if(isset($location['country'])){
				$country = $location['country'];
				if(in_array($country, $allowed_countries)){ //If country is in the list of allowed countries
					$allowed = true;
				}
			}
		}
		return $allowed;
	}

	/**
	 * Check what is the source of the saved abandoned cart and return it
	 * Source types NULL = checkout, 1 = Exit Intent, 2 = Early capture
	 * Variable $first - holds information weather this is the first time we are running this function or not (necessary because we need to know if the consent has to be saved now or it has already been saved before)
	 *
	 * @since    9.2
	 * @return 	 Array
	 */
	function get_cart_source(){
		$source = '';
		$first = true;
		if(WC()->session){ //If session exists
			if(!WC()->session->get('cartbounty_pro_saved_via')){ //If we do not know how the cart was initially saved
				if(isset($_POST['source'])){ //Check if data coming from a source or not
					switch ( $_POST['source'] ) {
						case 'cartbounty_pro_exit_intent':
							$source = 1;
							break;

						case 'cartbounty_pro_early_capture':
							$source = 2;
							break;
					}
					if(!empty($source)){
						WC()->session->set('cartbounty_pro_saved_via', $source);
					}
				}
				
			}else{ //In case we already know how the cart was initially saved - retrieve the information from session
				$source = WC()->session->get('cartbounty_pro_saved_via');
				$first = false;
			}
		}
		return array(
			'source' 	=> $source,
			'first'		=> $first
		);
	}

	/**
	 * Retrieve abandoned cart data value from database of a given key (e.g. name, surname, phone, email..)
	 *
	 * @since    9.7
	 * @return   string
	 * @param    string 	$value    		Key value that must be returned 
	 */
	function get_saved_cart_data( $value ){
		global $wpdb;
		$cart = $this->read_cart();
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;

		$result = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT $value
				FROM $cart_table
				WHERE session_id = %s",
				$cart['session_id']
			)
		);

		if( !$result ){
			$result = false;
		}

		return $result;
	}

	/**
	 * Get product count isnide shopping cart
	 *
	 * @since    9.8
	 * @return   integer
	 */
	function get_cart_product_count(){
		$count = 0;

		if( WC()->cart ){
			$count = WC()->cart->get_cart_contents_count();
		}

		return $count;
	}
}